@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright (c) 2017 Satpy developers
+# Copyright (c) 2017-2019 Satpy developers
 #
 # This file is part of satpy.
 #
@@ -15,24 +15,6 @@
 #
 # You should have received a copy of the GNU General Public License along with
 # satpy.  If not, see <http://www.gnu.org/licenses/>.
-# Author(s):
-
-#   Lorenzo Clementi <lorenzo.clementi@meteoswiss.ch>
-#   Panu Lahtinen <panu.lahtinen@fmi.fi>
-
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
-
 """
 Reader for generic image (e.g. gif, png, jpg, tif, geotiff, ...).
 
@@ -41,6 +23,7 @@ available in the file (eg. geotiff).
 """
 
 import logging
+import rasterio
 
 import xarray as xr
 import dask.array as da
@@ -48,6 +31,7 @@ import numpy as np
 
 from satpy.readers.file_handlers import BaseFileHandler
 from satpy import CHUNK_SIZE
+from pyresample import utils
 
 BANDS = {1: ['L'],
          2: ['L', 'A'],
@@ -58,8 +42,10 @@ logger = logging.getLogger(__name__)
 
 
 class GenericImageFileHandler(BaseFileHandler):
+    """Handle reading of generic image files."""
 
     def __init__(self, filename, filename_info, filetype_info):
+        """Initialize filehandler."""
         super(GenericImageFileHandler, self).__init__(
             filename, filename_info, filetype_info)
         self.finfo = filename_info
@@ -73,13 +59,15 @@ class GenericImageFileHandler(BaseFileHandler):
         self.read()
 
     def read(self):
-        """Read the image"""
-        data = xr.open_rasterio(self.finfo["filename"],
-                                chunks=(1, CHUNK_SIZE, CHUNK_SIZE))
-        attrs = data.attrs.copy()
+        """Read the image."""
+        dataset = rasterio.open(self.finfo['filename'])
+
         # Create area definition
-        if hasattr(data, 'crs'):
-            self.area = self.get_geotiff_area_def(data.crs)
+        if hasattr(dataset, 'crs') and dataset.crs is not None:
+            self.area = utils.get_area_def_from_raster(dataset)
+
+        data = xr.open_rasterio(dataset, chunks=(1, CHUNK_SIZE, CHUNK_SIZE))
+        attrs = data.attrs.copy()
 
         # Rename to Satpy convention
         data = data.rename({'band': 'bands'})
@@ -97,20 +85,19 @@ class GenericImageFileHandler(BaseFileHandler):
         self.file_content['image'] = data
 
     def get_area_def(self, dsid):
+        """Get area definition of the image."""
         if self.area is None:
             raise NotImplementedError("No CRS information available from image")
         return self.area
 
-    def get_geotiff_area_def(self, crs):
-        """Get extents from GeoTIFF file"""
-        return get_geotiff_area_def(self.finfo['filename'], crs)
-
     @property
     def start_time(self):
+        """Return start time."""
         return self.finfo['start_time']
 
     @property
     def end_time(self):
+        """Return end time."""
         return self.finfo['end_time']
 
     def get_dataset(self, key, info):
@@ -119,27 +106,6 @@ class GenericImageFileHandler(BaseFileHandler):
         return self.file_content[key.name]
 
 
-def get_geotiff_area_def(filename, crs):
-    """Read area definition from a geotiff."""
-    from osgeo import gdal
-    from pyresample.geometry import AreaDefinition
-    from pyresample.utils import proj4_str_to_dict
-    fid = gdal.Open(filename)
-    geo_transform = fid.GetGeoTransform()
-    pcs_id = fid.GetProjection().split('"')[1]
-    min_x = geo_transform[0]
-    max_y = geo_transform[3]
-    x_size = fid.RasterXSize
-    y_size = fid.RasterYSize
-    max_x = min_x + geo_transform[1] * x_size
-    min_y = max_y + geo_transform[5] * y_size
-    area_extent = [min_x, min_y, max_x, max_y]
-    area_def = AreaDefinition('geotiff_area', pcs_id, pcs_id,
-                              proj4_str_to_dict(crs),
-                              x_size, y_size, area_extent)
-    return area_def
-
-
 def mask_image_data(data):
     """Mask image data if alpha channel is present."""
     if data.bands.size in (2, 4):
