@@ -1,21 +1,20 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright (c) 2018 PyTroll developers
-
+# Copyright (c) 2018 Satpy developers
+#
 # This file is part of satpy.
-
+#
 # satpy is free software: you can redistribute it and/or modify it under the
 # terms of the GNU General Public License as published by the Free Software
 # Foundation, either version 3 of the License, or (at your option) any later
 # version.
-
+#
 # satpy is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 # A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License along with
 # satpy.  If not, see <http://www.gnu.org/licenses/>.
-
 """Reader for GOES 8-15 imager data in netCDF format from NOAA CLASS
    Also handles GOES 15 data in netCDF format reformated by Eumetsat
 
@@ -66,91 +65,123 @@ be obtained by calibrating all possible counts with both the minimum and the
 maximum calibration coefficients and computing the difference. The maximum
 differences are:
 
+======= ===== ====
 GOES-8
-======
-00_7 0.0   %  # Counts are normalized
-03_9 0.187 K
-06_8 0.0   K  # only one detector
-10_7 0.106 K
-12_0 0.036 K
+------------------
+Channel Diff  Unit
+======= ===== ====
+00_7    0.0   %  # Counts are normalized
+03_9    0.187 K
+06_8    0.0   K  # only one detector
+10_7    0.106 K
+12_0    0.036 K
+======= ===== ====
 
+======= ===== ====
 GOES-9
-========
-00_7 0.0   %  # Counts are normalized
-03_9 0.0   K  # coefs identical
-06_8 0.0   K  # only one detector
-10_7 0.021 K
-12_0 0.006 K
+------------------
+Channel Diff  Unit
+======= ===== ====
+00_7    0.0   %  # Counts are normalized
+03_9    0.0   K  # coefs identical
+06_8    0.0   K  # only one detector
+10_7    0.021 K
+12_0    0.006 K
+======= ===== ====
 
+======= ===== ====
 GOES-10
-========
-00_7 1.05  %
-03_9 0.0   K  # coefs identical
-06_8 0.0   K  # only one detector
-10_7 0.013 K
-12_0 0.004 K
+------------------
+Channel Diff  Unit
+======= ===== ====
+00_7    1.05  %
+03_9    0.0   K  # coefs identical
+06_8    0.0   K  # only one detector
+10_7    0.013 K
+12_0    0.004 K
+======= ===== ====
 
+======= ===== ====
 GOES-11
-========
-00_7 1.25  %
-03_9 0.0   K  # coefs identical
-06_8 0.0   K  # only one detector
-10_7 0.0   K  # coefs identical
-12_0 0.065 K
+------------------
+Channel Diff  Unit
+======= ===== ====
+00_7    1.25  %
+03_9    0.0   K  # coefs identical
+06_8    0.0   K  # only one detector
+10_7    0.0   K  # coefs identical
+12_0    0.065 K
+======= ===== ====
 
+======= ===== ====
 GOES-12
-========
-00_7 0.8   %
-03_9 0.0   K  # coefs identical
-06_5 0.044 K
-10_7 0.0   K  # coefs identical
-13_3 0.0   K  # only one detector
+------------------
+Channel Diff  Unit
+======= ===== ====
+00_7    0.8   %
+03_9    0.0   K  # coefs identical
+06_5    0.044 K
+10_7    0.0   K  # coefs identical
+13_3    0.0   K  # only one detector
+======= ===== ====
 
+======= ===== ====
 GOES-13
-========
-00_7 1.31  %
-03_9 0.0   K  # coefs identical
-06_5 0.085 K
-10_7 0.008 K
-13_3 0.0   K  # only one detector
+------------------
+Channel Diff  Unit
+======= ===== ====
+00_7    1.31  %
+03_9    0.0   K  # coefs identical
+06_5    0.085 K
+10_7    0.008 K
+13_3    0.0   K  # only one detector
+======= ===== ====
 
+======= ===== ====
 GOES-14
-========
-00_7 0.66  %
-03_9 0.0   K  # coefs identical
-06_5 0.043 K
-10_7 0.006 K
-13_3 0.003 K
+------------------
+Channel Diff  Unit
+======= ===== ====
+00_7    0.66  %
+03_9    0.0   K  # coefs identical
+06_5    0.043 K
+10_7    0.006 K
+13_3    0.003 K
+======= ===== ====
 
+======= ===== ====
 GOES-15
-========
-00_7 0.86  %
-03_9 0.0   K  # coefs identical
-06_5 0.02  K
-10_7 0.009 K
-13_3 0.008 K
-
+------------------
+Channel Diff  Unit
+======= ===== ====
+00_7    0.86  %
+03_9    0.0   K  # coefs identical
+06_5    0.02  K
+10_7    0.009 K
+13_3    0.008 K
+======= ===== ====
 
 References:
 
-[GVAR] https://goes.gsfc.nasa.gov/text/GVARRDL98.pdf
-[BOOK-N] https://goes.gsfc.nasa.gov/text/GOES-N_Databook/databook.pdf
-[BOOK-I] https://goes.gsfc.nasa.gov/text/databook/databook.pdf
-[IR] https://www.ospo.noaa.gov/Operations/GOES/calibration/gvar-conversion.html
-[VIS] https://www.ospo.noaa.gov/Operations/GOES/calibration/goes-vis-ch-calibration.html
-[FAQ] https://www.ncdc.noaa.gov/sites/default/files/attachments/Satellite-Frequently-Asked-Questions_2.pdf
-[SCHED-W] http://www.ospo.noaa.gov/Operations/GOES/west/imager-routine.html
-[SCHED-E] http://www.ospo.noaa.gov/Operations/GOES/east/imager-routine.html
+- [GVAR] https://goes.gsfc.nasa.gov/text/GVARRDL98.pdf
+- [BOOK-N] https://goes.gsfc.nasa.gov/text/GOES-N_Databook/databook.pdf
+- [BOOK-I] https://goes.gsfc.nasa.gov/text/databook/databook.pdf
+- [IR] https://www.ospo.noaa.gov/Operations/GOES/calibration/gvar-conversion.html
+- [VIS] https://www.ospo.noaa.gov/Operations/GOES/calibration/goes-vis-ch-calibration.html
+- [FAQ] https://www.ncdc.noaa.gov/sites/default/files/attachments/Satellite-Frequently-Asked-Questions_2.pdf
+- [SCHED-W] http://www.ospo.noaa.gov/Operations/GOES/west/imager-routine.html
+- [SCHED-E] http://www.ospo.noaa.gov/Operations/GOES/east/imager-routine.html
 
 Eumetsat formated netCDF data:
 
 The main differences are:
-1: The geolocation is in a separate file, used for all bands
-2: VIS data is calibrated to Albedo (or reflectance)
-3: IR data is calibrated to radiance.
-4: VIS data is downsampled to IR resolution (4km)
-5: File name differs also slightly
-6: Data is received via EumetCast
+
+1. The geolocation is in a separate file, used for all bands
+2. VIS data is calibrated to Albedo (or reflectance)
+3. IR data is calibrated to radiance.
+4. VIS data is downsampled to IR resolution (4km)
+5. File name differs also slightly
+6. Data is received via EumetCast
 
 """
 
@@ -162,7 +193,6 @@ import re
 
 import numpy as np
 import xarray as xr
-import xarray.ufuncs as xu
 
 import pyresample.geometry
 from satpy import CHUNK_SIZE
@@ -619,7 +649,7 @@ class GOESNCBaseFileHandler(BaseFileHandler):
             Mask (1=earth, 0=space)
         """
         logger.debug('Computing earth mask')
-        return xu.fabs(lat) <= 90
+        return np.fabs(lat) <= 90
 
     @staticmethod
     def _get_nadir_pixel(earth_mask, sector):
@@ -686,14 +716,13 @@ class GOESNCBaseFileHandler(BaseFileHandler):
                 sampling = SAMPLING_NS_IR
             pix_size = ALTITUDE * sampling
             area_def = pyresample.geometry.AreaDefinition(
-                area_id='goes_geos_uniform',
-                name='{} geostationary projection (uniform sampling)'.format(
-                    self.platform_name),
-                proj_id='goes_geos_uniform',
-                proj_dict=proj_dict,
-                x_size=np.rint((urx - llx) / pix_size).astype(int),
-                y_size=np.rint((ury - lly) / pix_size).astype(int),
-                area_extent=area_extent)
+                'goes_geos_uniform',
+                '{} geostationary projection (uniform sampling)'.format(self.platform_name),
+                'goes_geos_uniform',
+                proj_dict,
+                np.rint((urx - llx) / pix_size).astype(int),
+                np.rint((ury - lly) / pix_size).astype(int),
+                area_extent)
 
             return area_def
         else:
@@ -842,11 +871,11 @@ class GOESNCBaseFileHandler(BaseFileHandler):
 
         # Compute brightness temperature using inverse Planck formula
         n = coefs['n']
-        bteff = C2 * n / xu.log(1 + C1 * n**3 / radiance.where(radiance > 0))
+        bteff = C2 * n / np.log(1 + C1 * n ** 3 / radiance.where(radiance > 0))
         bt = xr.DataArray(bteff * coefs['b'] + coefs['a'])
 
         # Apply BT threshold
-        return bt.where(xu.logical_and(bt >= coefs['btmin'],
+        return bt.where(np.logical_and(bt >= coefs['btmin'],
                                        bt <= coefs['btmax']))
 
     @staticmethod
@@ -903,12 +932,12 @@ class GOESNCBaseFileHandler(BaseFileHandler):
         if 'file_type' in data.attrs:
             data.attrs.pop('file_type')
 
-        # Metadata discovered from the file
+        # Metadata discovered from the file.
         data.attrs.update(
             {'platform_name': self.platform_name,
              'sensor': self.sensor,
              'sector': self.sector,
-             'yaw_flip': self.meta['yaw_flip']}
+             'orbital_parameters': {'yaw_flip': self.meta['yaw_flip']}}
         )
         if self.meta['lon0'] is not None:
             # Attributes only available for full disc images. YAML reader
@@ -921,6 +950,11 @@ class GOESNCBaseFileHandler(BaseFileHandler):
                  'nadir_col': self.meta['nadir_col'],
                  'area_def_uniform_sampling': self.meta['area_def_uni']}
             )
+            data.attrs['orbital_parameters'].update(
+                {'projection_longitude': self.meta['lon0'],
+                 'projection_latitude': self.meta['lat0'],
+                 'projection_altitude': ALTITUDE}
+            )
 
     def __del__(self):
         try:
@@ -928,6 +962,33 @@ class GOESNCBaseFileHandler(BaseFileHandler):
         except (AttributeError, IOError, OSError):
             pass
 
+    def available_datasets(self, configured_datasets=None):
+        """Update information for or add datasets provided by this file.
+
+        If this file handler can load a dataset then it will supplement the
+        dataset info with the resolution and possibly coordinate datasets
+        needed to load it. Otherwise it will continue passing the dataset
+        information down the chain.
+
+        See
+        :meth:`satpy.readers.file_handlers.BaseFileHandler.available_datasets`
+        for details.
+
+        """
+        res = self.resolution
+        # update previously configured datasets
+        for is_avail, ds_info in (configured_datasets or []):
+            if is_avail is not None:
+                yield is_avail, ds_info
+
+            matches = self.file_type_matches(ds_info['file_type'])
+            if matches and ds_info.get('resolution') != res:
+                new_info = ds_info.copy()
+                new_info['resolution'] = res
+                yield True, new_info
+            elif is_avail is None:
+                yield is_avail, ds_info
+
 
 class GOESNCFileHandler(GOESNCBaseFileHandler):
     """File handler for GOES Imager data in netCDF format"""
