@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016-2017 Satpy developers
+# Copyright (c) 2016-2017, 2019 Satpy developers
 #
 # This file is part of satpy.
 #
@@ -15,9 +15,8 @@
 #
 # You should have received a copy of the GNU General Public License along with
 # satpy.  If not, see <http://www.gnu.org/licenses/>.
-"""Helpers for reading hdf5-based files.
+"""Helpers for reading hdf5-based files."""
 
-"""
 import logging
 import h5py
 import numpy as np
@@ -33,13 +32,14 @@ LOG = logging.getLogger(__name__)
 
 
 class HDF5FileHandler(BaseFileHandler):
-    """Small class for inspecting a HDF5 file and retrieve its metadata/header data.
-    """
+    """Small class for inspecting a HDF5 file and retrieve its metadata/header data."""
 
     def __init__(self, filename, filename_info, filetype_info):
+        """Initialize file handler."""
         super(HDF5FileHandler, self).__init__(
             filename, filename_info, filetype_info)
         self.file_content = {}
+
         try:
             file_handle = h5py.File(self.filename, 'r')
         except IOError:
@@ -59,8 +59,23 @@ class HDF5FileHandler(BaseFileHandler):
                 self.file_content[fc_key] = np2str(value)
             except ValueError:
                 self.file_content[fc_key] = value
+            except AttributeError:
+                # A HDF5 reference ?
+                value = self.get_reference(name, key)
+                if value is None:
+                    LOG.warning("Value cannot be converted - skip setting attribute %s", fc_key)
+                else:
+                    self.file_content[fc_key] = value
+
+    def get_reference(self, name, key):
+        """Get reference."""
+        with h5py.File(self.filename, 'r') as hf:
+            if isinstance(hf[name].attrs[key], h5py.h5r.Reference):
+                ref_name = h5py.h5r.get_name(hf[name].attrs[key], hf.id)
+                return hf[ref_name][()]
 
     def collect_metadata(self, name, obj):
+        """Collect metadata."""
         if isinstance(obj, h5py.Dataset):
             self.file_content[name] = obj
             self.file_content[name + "/dtype"] = obj.dtype
@@ -68,6 +83,7 @@ class HDF5FileHandler(BaseFileHandler):
         self._collect_attrs(name, obj.attrs)
 
     def __getitem__(self, key):
+        """Get item for given key."""
         val = self.file_content[key]
         if isinstance(val, h5py.Dataset):
             # these datasets are closed and inaccessible when the file is closed, need to reopen
@@ -80,9 +96,11 @@ class HDF5FileHandler(BaseFileHandler):
         return val
 
     def __contains__(self, item):
+        """Get item from file content."""
         return item in self.file_content
 
     def get(self, item, default=None):
+        """Get item."""
         if item in self:
             return self[item]
         else:
