@@ -15,8 +15,7 @@
 #
 # You should have received a copy of the GNU General Public License along with
 # satpy.  If not, see <http://www.gnu.org/licenses/>.
-"""HRIT format reader for JMA data
-************************************
+"""HRIT format reader for JMA data.
 
 References:
     JMA HRIT - Mission Specific Implementation
@@ -30,11 +29,11 @@ from datetime import datetime
 import numpy as np
 import xarray as xr
 
-from pyresample import geometry
 from satpy.readers.hrit_base import (HRITFileHandler, ancillary_text,
                                      annotation_header, base_hdr_map,
                                      image_data_function)
-from .utils import get_geostationary_mask
+from satpy.readers._geos_area import get_area_definition, get_area_extent
+from satpy.readers.utils import get_geostationary_mask
 
 logger = logging.getLogger('hrit_jma')
 
@@ -152,7 +151,7 @@ class HRITJMAFileHandler(HRITFileHandler):
         self.area = self._get_area_def()
 
     def _get_platform(self):
-        """Get the platform name
+        """Get the platform name.
 
         The platform is not specified explicitly in JMA HRIT files. For
         segmented data it is not even specified in the filename. But it
@@ -175,6 +174,7 @@ class HRITJMAFileHandler(HRITFileHandler):
         References:
         [MTSAT] http://www.data.jma.go.jp/mscweb/notice/Himawari7_e.html
         [HIMAWARI] http://www.data.jma.go.jp/mscweb/en/himawari89/space_segment/sample_hrit.html
+
         """
         try:
             return PLATFORMS[self.projection_name]
@@ -184,13 +184,14 @@ class HRITJMAFileHandler(HRITFileHandler):
             return UNKNOWN_PLATFORM
 
     def _check_sensor_platform_consistency(self, sensor):
-        """Make sure sensor and platform are consistent
+        """Make sure sensor and platform are consistent.
 
         Args:
             sensor (str) : Sensor name from YAML dataset definition
 
         Raises:
             ValueError if they don't match
+
         """
         ref_sensor = SENSORS.get(self.platform, None)
         if ref_sensor and not sensor == ref_sensor:
@@ -199,7 +200,7 @@ class HRITJMAFileHandler(HRITFileHandler):
                          .format(sensor, self.platform))
 
     def _get_line_offset(self):
-        """Get line offset for the current segment
+        """Get line offset for the current segment.
 
         Read line offset from the file and adapt it to the current segment
         or half disk scan so that
@@ -229,41 +230,24 @@ class HRITJMAFileHandler(HRITFileHandler):
 
     def _get_area_def(self):
         """Get the area definition of the band."""
-        cfac = np.int32(self.mda['cfac'])
-        lfac = np.int32(self.mda['lfac'])
-        coff = np.float32(self.mda['coff'])
-        loff = self._get_line_offset()
-
-        a = self.mda['projection_parameters']['a']
-        b = self.mda['projection_parameters']['b']
-        h = self.mda['projection_parameters']['h']
-        lon_0 = self.mda['projection_parameters']['SSP_longitude']
-
-        nlines = int(self.mda['number_of_lines'])
-        ncols = int(self.mda['number_of_columns'])
-
-        area_extent = self.get_area_extent((nlines, ncols),
-                                           (loff, coff),
-                                           (lfac, cfac),
-                                           h)
-
-        proj_dict = {'a': float(a),
-                     'b': float(b),
-                     'lon_0': float(lon_0),
-                     'h': float(h),
-                     'proj': 'geos',
-                     'units': 'm'}
-
-        area = geometry.AreaDefinition(
-            AREA_NAMES[self.area_id]['short'],
-            AREA_NAMES[self.area_id]['long'],
-            'geosmsg',
-            proj_dict,
-            ncols,
-            nlines,
-            area_extent)
-
-        return area
+        pdict = {
+            'cfac': np.int32(self.mda['cfac']),
+            'lfac': np.int32(self.mda['lfac']),
+            'coff': np.float32(self.mda['coff']),
+            'loff': self._get_line_offset(),
+            'ncols': int(self.mda['number_of_columns']),
+            'nlines': int(self.mda['number_of_lines']),
+            'scandir': 'N2S',
+            'a': float(self.mda['projection_parameters']['a']),
+            'b': float(self.mda['projection_parameters']['b']),
+            'h': float(self.mda['projection_parameters']['h']),
+            'ssp_lon': float(self.mda['projection_parameters']['SSP_longitude']),
+            'a_name': AREA_NAMES[self.area_id]['short'],
+            'a_desc': AREA_NAMES[self.area_id]['long'],
+            'p_id': 'geosmsg'
+        }
+        area_extent = get_area_extent(pdict)
+        return get_area_definition(pdict, area_extent)
 
     def get_area_def(self, dsid):
         """Get the area definition of the band."""
@@ -295,7 +279,7 @@ class HRITJMAFileHandler(HRITFileHandler):
         return res
 
     def _mask_space(self, data):
-        """Mask space pixels"""
+        """Mask space pixels."""
         geomask = get_geostationary_mask(area=self.area)
         return data.where(geomask)
 
@@ -317,6 +301,6 @@ class HRITJMAFileHandler(HRITFileHandler):
             res = xr.DataArray(res,
                                dims=data.dims, attrs=data.attrs,
                                coords=data.coords)
-        res = res.where(data > 0)
+        res = res.where(data < 65535)
         logger.debug("Calibration time " + str(datetime.now() - tic))
         return res
