@@ -61,7 +61,7 @@ class LIFileHandler(BaseFileHandler):
     def end_time(self):
         return datetime.strptime(self.nc.attrs['end_time'], '%Y%m%d%H%M%S')
 
-    def get_dataset(self, key, info=None, out=None, xslice=None, yslice=None):
+    def get_dataset(self, key, info=None, out=None):
         """Load a dataset
         """
         if key in self.cache:
@@ -78,12 +78,6 @@ class LIFileHandler(BaseFileHandler):
         logger.debug("Key: {}".format(key.name))
         # Create reference grid
         grid = np.full((self.nlines, self.ncols), np.NaN)
-        # Set slices to full disc extent
-        if xslice is None:
-            xslice = slice(0, self.ncols, None)
-        if yslice is None:
-            yslice = slice(0, self.nlines, None)
-        logger.debug("Slices - x: {}, y: {}".format(xslice, yslice))
         # Get product values
         values = self.nc[typedict[key.name]]
         rows = self.nc['row']
@@ -98,21 +92,17 @@ class LIFileHandler(BaseFileHandler):
 
         # Correct for bottom left origin in LI row/column indices.
         rotgrid = np.flipud(grid)
-        logger.debug('Data shape: {}, {}'.format(yslice, xslice))
         # Rotate the grid by 90 degree clockwise
         rotgrid = np.rot90(rotgrid, 3)
         logger.warning("LI data has been rotated to fit to reference grid. \
                         Works only for test dataset")
-        # Slice the gridded lighting data
-        slicegrid = rotgrid[yslice, xslice]
         # Mask invalid values
-        ds = np.ma.masked_where(np.isnan(slicegrid), slicegrid)
+        ds = np.ma.masked_where(np.isnan(rotgrid), rotgrid)
         # Create dataset object
         out.data[:] = np.ma.getdata(ds)
         out.mask[:] = np.ma.getmask(ds)
         out.info.update(key.to_dict())
-
-        return(out)
+        return out
 
     def get_area_def(self, key, info=None):
         """Create AreaDefinition for specified product.
