@@ -15,13 +15,15 @@
 #
 # You should have received a copy of the GNU General Public License along with
 # satpy.  If not, see <http://www.gnu.org/licenses/>.
-"""Utilities and eventually also base classes for MSG HRIT/Native data reading
-"""
+"""Utilities and eventually also base classes for MSG HRIT/Native data reading."""
 
 import numpy as np
 from numpy.polynomial.chebyshev import Chebyshev
 import dask.array as da
 
+from satpy.readers.eum_base import (time_cds_short,
+                                    issue_revision)
+
 C1 = 1.19104273e-5
 C2 = 1.43877523
 
@@ -185,7 +187,7 @@ CALIB[324] = {'HRV': {'F': 79.0035 / np.pi},
 
 
 def get_cds_time(days, msecs):
-    """Compute timestamp given the days since epoch and milliseconds of the day
+    """Compute timestamp given the days since epoch and milliseconds of the day.
 
     1958-01-01 00:00 is interpreted as fill value and will be replaced by NaT (Not a Time).
 
@@ -197,6 +199,7 @@ def get_cds_time(days, msecs):
 
     Returns:
         numpy.datetime64: Timestamp(s)
+
     """
     if np.isscalar(days):
         days = np.array([days], dtype='int64')
@@ -253,17 +256,71 @@ def dec10216(inbuf):
     return arr16
 
 
+class MpefProductHeader(object):
+    """MPEF product header class."""
+
+    def get(self):
+        """Return numpy record_array for MPEF product header."""
+        record = [
+            ('MPEF_File_Id', np.int16),
+            ('MPEF_Header_Version', np.uint8),
+            ('ManualDissAuthRequest', np.bool),
+            ('ManualDisseminationAuth', np.bool),
+            ('DisseminationAuth', np.bool),
+            ('NominalTime', time_cds_short),
+            ('ProductQuality', np.uint8),
+            ('ProductCompleteness', np.uint8),
+            ('ProductTimeliness', np.uint8),
+            ('ProcessingInstanceId', np.int8),
+            ('ImagesUsed', self.images_used, (4,)),
+            ('BaseAlgorithmVersion',
+             issue_revision),
+            ('ProductAlgorithmVersion',
+             issue_revision),
+            ('InstanceServerName', 'S2'),
+            ('SpacecraftName', 'S2'),
+            ('Mission', 'S3'),
+            ('RectificationLongitude', 'S5'),
+            ('Encoding', 'S1'),
+            ('TerminationSpace', 'S1'),
+            ('EncodingVersion', np.uint16),
+            ('Channel', np.uint8),
+            ('Filler', 'S20'),
+            ('RepeatCycle', 'S15'),
+        ]
+
+        return np.dtype(record).newbyteorder('>')
+
+    @property
+    def images_used(self):
+        """Return structure for images_used."""
+        record = [
+            ('Padding1', 'S2'),
+            ('ExpectedImage', time_cds_short),
+            ('ImageReceived', np.bool),
+            ('Padding2', 'S1'),
+            ('UsedImageStart_Day', np.uint16),
+            ('UsedImageStart_Millsec', np.uint32),
+            ('Padding3', 'S2'),
+            ('UsedImageEnd_Day', np.uint16),
+            ('UsedImageEndt_Millsec', np.uint32),
+        ]
+
+        return record
+
+
+mpef_product_header = MpefProductHeader().get()
+
+
 class SEVIRICalibrationHandler(object):
-    """Calibration handler for SEVIRI HRIT- and native-formats.
-    """
+    """Calibration handler for SEVIRI HRIT- and native-formats."""
 
     def _convert_to_radiance(self, data, gain, offset):
         """Calibrate to radiance."""
-
         return (data * gain + offset).clip(0.0, None)
 
     def _erads2bt(self, data, channel_name):
-        """Computation based on effective radiance."""
+        """Convert effective radiance to brightness temperature."""
         cal_info = CALIB[self.platform_id][channel_name]
         alpha = cal_info["ALPHA"]
         beta = cal_info["BETA"]
@@ -283,7 +340,7 @@ class SEVIRICalibrationHandler(object):
             raise NotImplementedError('Unknown calibration type')
 
     def _srads2bt(self, data, channel_name):
-        """Computation based on spectral radiance."""
+        """Convert spectral radiance to brightness temperature."""
         a__, b__, c__ = BTFIT[channel_name]
         wavenumber = CALIB[self.platform_id][channel_name]["VC"]
         temp = self._tl15(data, wavenumber)
@@ -292,24 +349,22 @@ class SEVIRICalibrationHandler(object):
 
     def _tl15(self, data, wavenumber):
         """Compute the L15 temperature."""
-
         return ((C2 * wavenumber) /
                 np.log((1.0 / data) * C1 * wavenumber ** 3 + 1.0))
 
     def _vis_calibrate(self, data, solar_irradiance):
         """Calibrate to reflectance."""
-
         return data * 100.0 / solar_irradiance
 
 
 def chebyshev(coefs, time, domain):
-    """Evaluate a Chebyshev Polynomial
+    """Evaluate a Chebyshev Polynomial.
 
     Args:
         coefs (list, np.array): Coefficients defining the polynomial
         time (int, float): Time where to evaluate the polynomial
         domain (list, tuple): Domain (or time interval) for which the polynomial is defined: [left, right]
-
     Reference: Appendix A in the MSG Level 1.5 Image Data Format Description.
+
     """
     return Chebyshev(coefs, domain=domain)(time) - 0.5 * coefs[0]
