@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright (c) 2010-2018 Satpy developers
+# Copyright (c) 2010-2019 Satpy developers
 #
 # This file is part of satpy.
 #
@@ -15,8 +15,7 @@
 #
 # You should have received a copy of the GNU General Public License along with
 # satpy.  If not, see <http://www.gnu.org/licenses/>.
-"""SEVIRI HRIT format reader
-============================
+r"""SEVIRI HRIT format reader.
 
 Introduction
 ------------
@@ -42,6 +41,16 @@ Each image is decomposed into 24 segments (files) for the high-resolution-visibl
 visible (VIS) and infrared (IR) channels. Additionally there is one prologue and one epilogue file for the entire scan
 which contain global metadata valid for all channels.
 
+Reader Arguments
+----------------
+Some arguments can be provided to the reader to change it's behaviour. These are
+provided through the `Scene` instantiation, eg::
+
+  Scene(reader="seviri_l1b_hrit", filenames=fnames, reader_kwargs={'fill_hrv': False})
+
+To see the full list of arguments that can be provided, look into the documentation
+of `:class:HRITMSGFileHandler`.
+
 Example
 -------
 Here is an example how to read the data in satpy:
@@ -121,29 +130,33 @@ References:
 .. _Radiometric Calibration of MSG SEVIRI Level 1.5 Image Data in Equivalent Spectral Blackbody Radiance:
     https://www.eumetsat.int/website/wcm/idc/idcplg?IdcService=GET_FILE&dDocName=PDF_TEN_MSG_SEVIRI_RAD_CALIB&
     RevisionSelectionMethod=LatestReleased&Rendition=Web
+
 """
 
+from __future__ import division
+
 import copy
 import logging
 from datetime import datetime
 
+import dask.array as da
 import numpy as np
 import pyproj
+import xarray as xr
 
+import satpy.readers.utils as utils
 from pyresample import geometry
-
-from satpy.readers.eum_base import (time_cds_short,
-                                    recarray2dict)
+from satpy import CHUNK_SIZE
+from satpy.readers.eum_base import recarray2dict, time_cds_short
 from satpy.readers.hrit_base import (HRITFileHandler, ancillary_text,
                                      annotation_header, base_hdr_map,
                                      image_data_function)
-
-from satpy.readers.seviri_base import SEVIRICalibrationHandler, chebyshev, get_cds_time
-from satpy.readers.seviri_base import (CHANNEL_NAMES, VIS_CHANNELS, CALIB, SATNUM)
-
-from satpy.readers.seviri_l1b_native_hdr import (hrit_prologue, hrit_epilogue,
+from satpy.readers.seviri_base import (CALIB, CHANNEL_NAMES, SATNUM,
+                                       VIS_CHANNELS, SEVIRICalibrationHandler,
+                                       chebyshev, get_cds_time)
+from satpy.readers.seviri_l1b_native_hdr import (hrit_epilogue, hrit_prologue,
                                                  impf_configuration)
-import satpy.readers.utils as utils
+from satpy.readers._geos_area import get_area_extent, get_area_definition
 
 
 logger = logging.getLogger('hrit_msg')
@@ -201,29 +214,35 @@ cuc_time = np.dtype([('coarse', 'u1', (4, )),
 
 
 class NoValidOrbitParams(Exception):
+    """Exception when validOrbitParameters are missing."""
+
     pass
 
 
 class HRITMSGPrologueEpilogueBase(HRITFileHandler):
+    """Base reader for prologue and epilogue files."""
+
     def __init__(self, filename, filename_info, filetype_info, hdr_info):
+        """Initialize the file handler for prologue and epilogue files."""
         super(HRITMSGPrologueEpilogueBase, self).__init__(filename, filename_info, filetype_info, hdr_info)
         self._reduced = None
 
     def _reduce(self, mda, max_size):
+        """Reduce the metadata."""
         if self._reduced is None:
             self._reduced = utils.reduce_mda(mda, max_size=max_size)
         return self._reduced
 
     def reduce(self, max_size):
+        """Reduce the metadata (placeholder)."""
         raise NotImplementedError
 
 
 class HRITMSGPrologueFileHandler(HRITMSGPrologueEpilogueBase):
-    """SEVIRI HRIT prologue reader.
-    """
+    """SEVIRI HRIT prologue reader."""
 
     def __init__(self, filename, filename_info, filetype_info, calib_mode='nominal',
-                 ext_calib_coefs=None, mda_max_array_size=None):
+                 ext_calib_coefs=None, mda_max_array_size=None, fill_hrv=None):
         """Initialize the reader."""
         super(HRITMSGPrologueFileHandler, self).__init__(filename, filename_info,
                                                          filetype_info,
@@ -242,7 +261,6 @@ class HRITMSGPrologueFileHandler(HRITMSGPrologueEpilogueBase):
 
     def read_prologue(self):
         """Read the prologue metadata."""
-
         with open(self.filename, "rb") as fp_:
             fp_.seek(self.mda['total_header_length'])
             data = np.fromfile(fp_, dtype=hrit_prologue, count=1)
@@ -255,7 +273,7 @@ class HRITMSGPrologueFileHandler(HRITMSGPrologueEpilogueBase):
                 self.prologue.update(recarray2dict(impf))
 
     def get_satpos(self):
-        """Get actual satellite position in geodetic coordinates (WGS-84)
+        """Get actual satellite position in geodetic coordinates (WGS-84).
 
         Returns: Longitude [deg east], Latitude [deg north] and Altitude [m]
         """
@@ -281,7 +299,7 @@ class HRITMSGPrologueFileHandler(HRITMSGPrologueEpilogueBase):
         return self.satpos
 
     def _get_satpos_cart(self):
-        """Determine satellite position in earth-centered cartesion coordinates
+        """Determine satellite position in earth-centered cartesion coordinates.
 
         The coordinates as a function of time are encoded in the coefficients of an 8th-order Chebyshev polynomial.
         In the prologue there is one set of coefficients for each coordinate (x, y, z). The coordinates are obtained by
@@ -291,7 +309,7 @@ class HRITMSGPrologueFileHandler(HRITMSGPrologueEpilogueBase):
         """
         orbit_polynomial = self.prologue['SatelliteStatus']['Orbit']['OrbitPolynomial']
 
-        # Find Chebyshev coefficients for the given time
+        # Find Chebyshev coefficients for the start time of the scan
         coef_idx = self._find_orbit_coefs()
         tstart = orbit_polynomial['StartTime'][0, coef_idx]
         tend = orbit_polynomial['EndTime'][0, coef_idx]
@@ -309,30 +327,52 @@ class HRITMSGPrologueFileHandler(HRITMSGPrologueEpilogueBase):
         return x*1000, y*1000, z*1000  # km -> m
 
     def _find_orbit_coefs(self):
-        """Find orbit coefficients for the current time
+        """Find orbit coefficients for the start time of the scan.
 
-        The orbital Chebyshev coefficients are only valid for a certain time interval. The header entry
-        SatelliteStatus/Orbit/OrbitPolynomial contains multiple coefficients for multiple time intervals. Find the
-        coefficients which are valid for the nominal timestamp of the scan.
+        The header entry SatelliteStatus/Orbit/OrbitPolynomial contains multiple coefficients, each
+        of them valid for a certain time interval. Find the coefficients which are valid for the
+        start time of the scan.
+
+        A manoeuvre is a discontinuity in the orbit parameters. The flight dynamic algorithms are
+        not made to interpolate over the time-span of the manoeuvre; hence we have elements
+        describing the orbit before a manoeuvre and a new set of elements describing the orbit after
+        the manoeuvre. The flight dynamic products are created so that there is an intentional gap
+        at the time of the manoeuvre. Also the two pre-manoeuvre elements may overlap. But the
+        overlap is not of an issue as both sets of elements describe the same pre-manoeuvre orbit
+        (with negligible variations).
 
         Returns: Corresponding index in the coefficient list.
+
         """
-        # Find index of interval enclosing the nominal timestamp of the scan
         time = np.datetime64(self.prologue['ImageAcquisition']['PlannedAcquisitionTime']['TrueRepeatCycleStart'])
         intervals_tstart = self.prologue['SatelliteStatus']['Orbit']['OrbitPolynomial']['StartTime'][0].astype(
             'datetime64[us]')
         intervals_tend = self.prologue['SatelliteStatus']['Orbit']['OrbitPolynomial']['EndTime'][0].astype(
             'datetime64[us]')
         try:
-            return np.where(np.logical_and(time >= intervals_tstart, time < intervals_tend))[0][0]
-        except IndexError:
-            raise NoValidOrbitParams('Unable to find orbit coefficients valid for {}'.format(time))
+            # Find index of interval enclosing the nominal timestamp of the scan. If there are
+            # multiple enclosing intervals, use the most recent one.
+            enclosing = np.where(np.logical_and(time >= intervals_tstart, time < intervals_tend))[0]
+            most_recent = np.argmax(intervals_tstart[enclosing])
+            return enclosing[most_recent]
+        except ValueError:
+            # No enclosing interval. Instead, find the interval whose centre is closest to the scan's timestamp
+            # (but not more than 6 hours apart)
+            intervals_centre = intervals_tstart + 0.5 * (intervals_tend - intervals_tstart)
+            diffs_us = (time - intervals_centre).astype('i8')
+            closest_match = np.argmin(np.fabs(diffs_us))
+            if abs(intervals_centre[closest_match] - time) < np.timedelta64(6, 'h'):
+                logger.warning('No orbit coefficients valid for {}. Using closest match.'.format(time))
+                return closest_match
+            else:
+                raise NoValidOrbitParams('Unable to find orbit coefficients valid for {}'.format(time))
 
     def get_earth_radii(self):
-        """Get earth radii from prologue
+        """Get earth radii from prologue.
 
         Returns:
             Equatorial radius, polar radius [m]
+
         """
         earth_model = self.prologue['GeometricProcessing']['EarthModel']
         a = earth_model['EquatorialRadius'] * 1000
@@ -341,15 +381,15 @@ class HRITMSGPrologueFileHandler(HRITMSGPrologueEpilogueBase):
         return a, b
 
     def reduce(self, max_size):
+        """Reduce the prologue metadata."""
         return self._reduce(self.prologue, max_size=max_size)
 
 
 class HRITMSGEpilogueFileHandler(HRITMSGPrologueEpilogueBase):
-    """SEVIRI HRIT epilogue reader.
-    """
+    """SEVIRI HRIT epilogue reader."""
 
     def __init__(self, filename, filename_info, filetype_info, calib_mode='nominal',
-                 ext_calib_coefs=None, mda_max_array_size=None):
+                 ext_calib_coefs=None, mda_max_array_size=None, fill_hrv=None):
         """Initialize the reader."""
         super(HRITMSGEpilogueFileHandler, self).__init__(filename, filename_info,
                                                          filetype_info,
@@ -367,18 +407,18 @@ class HRITMSGEpilogueFileHandler(HRITMSGPrologueEpilogueBase):
 
     def read_epilogue(self):
         """Read the epilogue metadata."""
-
         with open(self.filename, "rb") as fp_:
             fp_.seek(self.mda['total_header_length'])
             data = np.fromfile(fp_, dtype=hrit_epilogue, count=1)
             self.epilogue.update(recarray2dict(data))
 
     def reduce(self, max_size):
+        """Reduce the epilogue metadata."""
         return self._reduce(self.epilogue, max_size=max_size)
 
 
 class HRITMSGFileHandler(HRITFileHandler, SEVIRICalibrationHandler):
-    """SEVIRI HRIT format reader
+    """SEVIRI HRIT format reader.
 
     **Calibration**
 
@@ -431,16 +471,27 @@ class HRITMSGFileHandler(HRITFileHandler, SEVIRICalibrationHandler):
 
     By default, arrays with more than 100 elements are excluded from the raw reader metadata to
     limit memory usage. This threshold can be adjusted using the `mda_max_array_size` keyword
-    argument:
+    argument::
 
         scene = satpy.Scene(filenames,
                             reader='seviri_l1b_hrit',
                             reader_kwargs={'mda_max_array_size': 1000})
 
+    **Padding of the HRV channel**
+
+    By default, the HRV channel is loaded padded with no-data, that is it is
+    returned as a full-disk dataset. If you want the original, unpadded, data,
+    just provide the `fill_hrv` as False in the `reader_kwargs`::
+
+        scene = satpy.Scene(filenames,
+                            reader='seviri_l1b_hrit',
+                            reader_kwargs={'fill_hrv': False})
+
     """
+
     def __init__(self, filename, filename_info, filetype_info,
                  prologue, epilogue, calib_mode='nominal',
-                 ext_calib_coefs=None, mda_max_array_size=100):
+                 ext_calib_coefs=None, mda_max_array_size=100, fill_hrv=True):
         """Initialize the reader."""
         super(HRITMSGFileHandler, self).__init__(filename, filename_info,
                                                  filetype_info,
@@ -455,6 +506,7 @@ class HRITMSGFileHandler(HRITFileHandler, SEVIRICalibrationHandler):
         self._filename_info = filename_info
         self.ext_calib_coefs = ext_calib_coefs if ext_calib_coefs is not None else {}
         self.mda_max_array_size = mda_max_array_size
+        self.fill_hrv = fill_hrv
         calib_mode_choices = ('NOMINAL', 'GSICS')
         if calib_mode.upper() not in calib_mode_choices:
             raise ValueError('Invalid calibration mode: {}. Choose one of {}'.format(
@@ -464,8 +516,7 @@ class HRITMSGFileHandler(HRITFileHandler, SEVIRICalibrationHandler):
         self._get_header()
 
     def _get_header(self):
-        """Read the header info, and fill the metadata dictionary"""
-
+        """Read the header info, and fill the metadata dictionary."""
         earth_model = self.prologue['GeometricProcessing']['EarthModel']
         self.mda['offset_corrected'] = earth_model['TypeOfEarthModel'] == 2
 
@@ -483,9 +534,10 @@ class HRITMSGFileHandler(HRITFileHandler, SEVIRICalibrationHandler):
         self.mda['orbital_parameters']['satellite_nominal_longitude'] = self.prologue['SatelliteStatus'][
             'SatelliteDefinition']['NominalLongitude']
         self.mda['orbital_parameters']['satellite_nominal_latitude'] = 0.0
-        self.mda['orbital_parameters']['satellite_actual_longitude'] = actual_lon
-        self.mda['orbital_parameters']['satellite_actual_latitude'] = actual_lat
-        self.mda['orbital_parameters']['satellite_actual_altitude'] = actual_alt
+        if actual_lon is not None:
+            self.mda['orbital_parameters']['satellite_actual_longitude'] = actual_lon
+            self.mda['orbital_parameters']['satellite_actual_latitude'] = actual_lat
+            self.mda['orbital_parameters']['satellite_actual_altitude'] = actual_alt
 
         # Misc
         self.platform_id = self.prologue["SatelliteStatus"][
@@ -501,29 +553,17 @@ class HRITMSGFileHandler(HRITFileHandler, SEVIRICalibrationHandler):
 
     @property
     def start_time(self):
-
+        """Get the start time."""
         return self.epilogue['ImageProductionStats'][
             'ActualScanningSummary']['ForwardScanStart']
 
     @property
     def end_time(self):
-
+        """Get the end time."""
         return self.epilogue['ImageProductionStats'][
             'ActualScanningSummary']['ForwardScanEnd']
 
-    def get_xy_from_linecol(self, line, col, offsets, factors):
-        """Get the intermediate coordinates from line & col.
-
-        Intermediate coordinates are actually the instruments scanning angles.
-        """
-        loff, coff = offsets
-        lfac, cfac = factors
-        x__ = (col - coff) / cfac * 2**16
-        y__ = - (line - loff) / lfac * 2**16
-
-        return x__, y__
-
-    def get_area_extent(self, size, offsets, factors, platform_height):
+    def _get_area_extent(self, pdict):
         """Get the area extent of the file.
 
         Until December 2017, the data is shifted by 1.5km SSP North and West against the nominal GEOS projection. Since
@@ -534,23 +574,7 @@ class HRITMSGFileHandler(HRITFileHandler, SEVIRICalibrationHandler):
         of the area extent is documented in a `developer's memo <https://github.com/pytroll/satpy/wiki/
         SEVIRI-georeferencing-offset-correction>`_.
         """
-        nlines, ncols = size
-        h = platform_height
-
-        loff, coff = offsets
-        loff -= nlines
-        offsets = loff, coff
-        # count starts at 1
-        cols = 1 - 0.5
-        lines = 0.5 - 1
-        ll_x, ll_y = self.get_xy_from_linecol(-lines, cols, offsets, factors)
-
-        cols += ncols
-        lines += nlines
-        ur_x, ur_y = self.get_xy_from_linecol(-lines, cols, offsets, factors)
-
-        aex = (np.deg2rad(ll_x) * h, np.deg2rad(ll_y) * h,
-               np.deg2rad(ur_x) * h, np.deg2rad(ur_y) * h)
+        aex = get_area_extent(pdict)
 
         if not self.mda['offset_corrected']:
             # Geo-referencing offset present. Adjust area extent to match the shifted data. Note that we have to adjust
@@ -568,79 +592,90 @@ class HRITMSGFileHandler(HRITFileHandler, SEVIRICalibrationHandler):
 
     def get_area_def(self, dsid):
         """Get the area definition of the band."""
-        if dsid.name != 'HRV':
-            return super(HRITMSGFileHandler, self).get_area_def(dsid)
-
-        cfac = np.int32(self.mda['cfac'])
-        lfac = np.int32(self.mda['lfac'])
-        loff = np.float32(self.mda['loff'])
-
-        a = self.mda['projection_parameters']['a']
-        b = self.mda['projection_parameters']['b']
-        h = self.mda['projection_parameters']['h']
-        lon_0 = self.mda['projection_parameters']['SSP_longitude']
 
+        # Common parameters for both HRV and other channels
         nlines = int(self.mda['number_of_lines'])
-        ncols = int(self.mda['number_of_columns'])
+        loff = np.float32(self.mda['loff'])
+        pdict = {}
+        pdict['cfac'] = np.int32(self.mda['cfac'])
+        pdict['lfac'] = np.int32(self.mda['lfac'])
+        pdict['coff'] = np.float32(self.mda['coff'])
+
+        pdict['a'] = self.mda['projection_parameters']['a']
+        pdict['b'] = self.mda['projection_parameters']['b']
+        pdict['h'] = self.mda['projection_parameters']['h']
+        pdict['ssp_lon'] = self.mda['projection_parameters']['SSP_longitude']
+
+        pdict['nlines'] = nlines
+        pdict['ncols'] = int(self.mda['number_of_columns'])
+        if (self.prologue['ImageDescription']['Level15ImageProduction']
+                         ['ImageProcDirection'] == 0):
+            pdict['scandir'] = 'N2S'
+        else:
+            pdict['scandir'] = 'S2N'
+
+        # Compute area definition for non-HRV channels:
+        if dsid.name != 'HRV':
+            pdict['loff'] = loff - nlines
+            aex = self._get_area_extent(pdict)
+            pdict['a_name'] = 'geosmsg'
+            pdict['a_desc'] = 'MSG/SEVIRI low resolution channel area'
+            pdict['p_id'] = 'msg_lowres'
+            area = get_area_definition(pdict, aex)
+            self.area = area
+            return self.area
 
         segment_number = self.mda['segment_sequence_number']
 
-        current_first_line = (segment_number -
-                              self.mda['planned_start_segment_number']) * nlines
-        bounds = self.epilogue['ImageProductionStats']['ActualL15CoverageHRV']
+        current_first_line = ((segment_number -
+                               self.mda['planned_start_segment_number'])
+                              * pdict['nlines'])
+
+        # Or, if we are processing HRV:
+        pdict['a_name'] = 'geosmsg_hrv'
+        pdict['p_id'] = 'msg_hires'
+        bounds = self.epilogue['ImageProductionStats']['ActualL15CoverageHRV'].copy()
+        if self.fill_hrv:
+            bounds['UpperEastColumnActual'] = 1
+            bounds['UpperWestColumnActual'] = 11136
+            bounds['LowerEastColumnActual'] = 1
+            bounds['LowerWestColumnActual'] = 11136
+            pdict['ncols'] = 11136
 
         upper_south_line = bounds[
             'LowerNorthLineActual'] - current_first_line - 1
-        upper_south_line = min(max(upper_south_line, 0), nlines)
-
+        upper_south_line = min(max(upper_south_line, 0), pdict['nlines'])
         lower_coff = (5566 - bounds['LowerEastColumnActual'] + 1)
         upper_coff = (5566 - bounds['UpperEastColumnActual'] + 1)
 
-        lower_area_extent = self.get_area_extent((upper_south_line, ncols),
-                                                 (loff, lower_coff),
-                                                 (lfac, cfac),
-                                                 h)
+        # First we look at the lower window
+        pdict['nlines'] = upper_south_line
+        pdict['loff'] = loff - upper_south_line
+        pdict['coff'] = lower_coff
+        pdict['a_desc'] = 'MSG/SEVIRI high resolution channel, lower window'
+        lower_area_extent = self._get_area_extent(pdict)
+        lower_area = get_area_definition(pdict, lower_area_extent)
 
-        upper_area_extent = self.get_area_extent((nlines - upper_south_line,
-                                                  ncols),
-                                                 (loff - upper_south_line,
-                                                  upper_coff),
-                                                 (lfac, cfac),
-                                                 h)
-
-        proj_dict = {'a': float(a),
-                     'b': float(b),
-                     'lon_0': float(lon_0),
-                     'h': float(h),
-                     'proj': 'geos',
-                     'units': 'm'}
-
-        lower_area = geometry.AreaDefinition(
-            'some_area_name',
-            "On-the-fly area",
-            'geosmsg',
-            proj_dict,
-            ncols,
-            upper_south_line,
-            lower_area_extent)
-
-        upper_area = geometry.AreaDefinition(
-            'some_area_name',
-            "On-the-fly area",
-            'geosmsg',
-            proj_dict,
-            ncols,
-            nlines - upper_south_line,
-            upper_area_extent)
+        # Now the upper window
+        pdict['nlines'] = nlines - upper_south_line
+        pdict['loff'] = loff - pdict['nlines'] - upper_south_line
+        pdict['coff'] = upper_coff
+        pdict['a_desc'] = 'MSG/SEVIRI high resolution channel, upper window'
+        upper_area_extent = self._get_area_extent(pdict)
+        upper_area = get_area_definition(pdict, upper_area_extent)
 
         area = geometry.StackedAreaDefinition(lower_area, upper_area)
 
         self.area = area.squeeze()
-        return area
+        return self.area
 
     def get_dataset(self, key, info):
+        """Get the dataset."""
         res = super(HRITMSGFileHandler, self).get_dataset(key, info)
         res = self.calibrate(res, key.calibration)
+        if key.name == 'HRV' and self.fill_hrv:
+            res = self.pad_hrv_data(res)
+
         res.attrs['units'] = info['units']
         res.attrs['wavelength'] = info['wavelength']
         res.attrs['standard_name'] = info['standard_name']
@@ -665,6 +700,39 @@ class HRITMSGFileHandler(HRITFileHandler, SEVIRICalibrationHandler):
 
         return res
 
+    def pad_hrv_data(self, res):
+        """Add empty pixels around the HRV."""
+        logger.debug('Padding HRV data to full disk')
+        nlines = int(self.mda['number_of_lines'])
+
+        segment_number = self.mda['segment_sequence_number']
+
+        current_first_line = (segment_number
+                              - self.mda['planned_start_segment_number']) * nlines
+        bounds = self.epilogue['ImageProductionStats']['ActualL15CoverageHRV']
+
+        upper_south_line = bounds[
+          'LowerNorthLineActual'] - current_first_line - 1
+        upper_south_line = min(max(upper_south_line, 0), nlines)
+
+        data_list = list()
+        if upper_south_line > 0:
+            # we have some of the lower window
+            data_lower = pad_data(res[:upper_south_line, :].data,
+                                  (upper_south_line, 11136),
+                                  bounds['LowerEastColumnActual'],
+                                  bounds['LowerWestColumnActual'])
+            data_list.append(data_lower)
+
+        if upper_south_line < nlines:
+            # we have some of the upper window
+            data_upper = pad_data(res[upper_south_line:, :].data,
+                                  (nlines - upper_south_line, 11136),
+                                  bounds['UpperEastColumnActual'],
+                                  bounds['UpperWestColumnActual'])
+            data_list.append(data_upper)
+        return xr.DataArray(da.vstack(data_list), dims=('y', 'x'))
+
     def calibrate(self, data, calibration):
         """Calibrate the data."""
         tic = datetime.now()
@@ -712,7 +780,7 @@ class HRITMSGFileHandler(HRITFileHandler, SEVIRICalibrationHandler):
         return res
 
     def _get_raw_mda(self):
-        """Compile raw metadata to be included in the dataset attributes"""
+        """Compile raw metadata to be included in the dataset attributes."""
         # Metadata from segment header (excluding items which vary among the different segments)
         raw_mda = copy.deepcopy(self.mda)
         for key in ('image_segment_line_quality', 'segment_sequence_number', 'annotation_header', 'loff'):
@@ -725,18 +793,21 @@ class HRITMSGFileHandler(HRITFileHandler, SEVIRICalibrationHandler):
         return raw_mda
 
     def _get_timestamps(self):
-        """Read scanline timestamps from the segment header"""
+        """Read scanline timestamps from the segment header."""
         tline = self.mda['image_segment_line_quality']['line_mean_acquisition']
         return get_cds_time(days=tline['days'], msecs=tline['milliseconds'])
 
 
-def show(data, negate=False):
-    """Show the stretched data.
-    """
-    from PIL import Image as pil
-    data = np.array((data - data.min()) * 255.0 /
-                    (data.max() - data.min()), np.uint8)
-    if negate:
-        data = 255 - data
-    img = pil.fromarray(data)
-    img.show()
+def pad_data(data, final_size, east_bound, west_bound):
+    """Pad the data given east and west bounds and the desired size."""
+    nlines = final_size[0]
+    if west_bound - east_bound != data.shape[1] - 1:
+        raise IndexError('East and west bounds do not match data shape')
+    padding_east = da.zeros((nlines, east_bound - 1),
+                            dtype=data.dtype, chunks=CHUNK_SIZE)
+    padding_west = da.zeros((nlines, (final_size[1] - west_bound)),
+                            dtype=data.dtype, chunks=CHUNK_SIZE)
+    if np.issubdtype(data.dtype, np.floating):
+        padding_east = padding_east * np.nan
+        padding_west = padding_west * np.nan
+    return np.hstack((padding_east, data, padding_west))
