@@ -22,14 +22,24 @@ import logging
 from contextlib import closing
 import tempfile
 import bz2
+import sys
 import os
+import shutil
 import numpy as np
 import pyproj
+from six import BytesIO
+from subprocess import Popen, PIPE
 from pyresample.geometry import AreaDefinition
 from pyresample.boundary import AreaDefBoundary, Boundary
 
 from satpy import CHUNK_SIZE
 
+try:
+    from shutil import which
+except ImportError:
+    # python 2 - won't be used, but needed for mocking in tests
+    which = None
+
 LOGGER = logging.getLogger(__name__)
 
 
@@ -45,8 +55,9 @@ def np2str(value):
 
     """
     if hasattr(value, 'dtype') and \
-            issubclass(value.dtype.type, (np.string_, np.object_)) and value.size == 1:
-        value = np.asscalar(value)
+            issubclass(value.dtype.type, (np.string_, np.object_)) \
+            and value.size == 1:
+        value = value.item()
         if not isinstance(value, str):
             # python 3 - was scalar numpy array of bytes
             # otherwise python 2 - scalar numpy array of 'str'
@@ -61,9 +72,9 @@ def get_geostationary_angle_extent(geos_area):
     # TODO: take into account sweep_axis_angle parameter
 
     # get some projection parameters
-    req = geos_area.proj_dict['a'] / 1000
-    rp = geos_area.proj_dict['b'] / 1000
-    h = geos_area.proj_dict['h'] / 1000 + req
+    req = float(geos_area.proj_dict['a']) / 1000
+    rp = float(geos_area.proj_dict['b']) / 1000
+    h = float(geos_area.proj_dict['h']) / 1000 + req
 
     # compute some constants
     aeq = 1 - req**2 / (h ** 2)
@@ -77,7 +88,7 @@ def get_geostationary_angle_extent(geos_area):
 
 
 def get_geostationary_mask(area):
-    """Compute a mask of the earth's shape as seen by a geostationary satellite
+    """Compute a mask of the earth's shape as seen by a geostationary satellite.
 
     Args:
         area (pyresample.geometry.AreaDefinition) : Corresponding area
@@ -85,6 +96,7 @@ def get_geostationary_mask(area):
 
     Returns:
         Boolean mask, True inside the earth's shape, False outside.
+
     """
     # Compute projection coordinates at the earth's limb
     h = area.proj_dict['h']
@@ -101,12 +113,12 @@ def get_geostationary_mask(area):
 
 def _lonlat_from_geos_angle(x, y, geos_area):
     """Get lons and lats from x, y in projection coordinates."""
-    h = (geos_area.proj_dict['h'] + geos_area.proj_dict['a']) / 1000
-    b__ = (geos_area.proj_dict['a'] / geos_area.proj_dict['b']) ** 2
+    h = float(geos_area.proj_dict['h'] + geos_area.proj_dict['a']) / 1000
+    b__ = (geos_area.proj_dict['a'] / float(geos_area.proj_dict['b'])) ** 2
 
     sd = np.sqrt((h * np.cos(x) * np.cos(y)) ** 2 -
                  (np.cos(y)**2 + b__ * np.sin(y)**2) *
-                 (h**2 - (geos_area.proj_dict['a'] / 1000)**2))
+                 (h**2 - (float(geos_area.proj_dict['a']) / 1000)**2))
     # sd = 0
 
     sn = (h * np.cos(x) * np.cos(y) - sd) / (np.cos(y)**2 + b__ * np.sin(y)**2)
@@ -126,17 +138,18 @@ def get_geostationary_bounding_box(geos_area, nb_points=50):
 
     Args:
       nb_points: Number of points on the polygon
+
     """
     xmax, ymax = get_geostationary_angle_extent(geos_area)
 
     # generate points around the north hemisphere in satellite projection
     # make it a bit smaller so that we stay inside the valid area
-    x = np.cos(np.linspace(-np.pi, 0, nb_points / 2)) * (xmax - 0.001)
-    y = -np.sin(np.linspace(-np.pi, 0, nb_points / 2)) * (ymax - 0.001)
+    x = np.cos(np.linspace(-np.pi, 0, nb_points // 2)) * (xmax - 0.001)
+    y = -np.sin(np.linspace(-np.pi, 0, nb_points // 2)) * (ymax - 0.001)
 
     # clip the projection coordinates to fit the area extent of geos_area
     ll_x, ll_y, ur_x, ur_y = (np.array(geos_area.area_extent) /
-                              geos_area.proj_dict['h'])
+                              float(geos_area.proj_dict['h']))
 
     x = np.clip(np.concatenate([x, x[::-1]]), min(ll_x, ur_x), max(ll_x, ur_x))
     y = np.clip(np.concatenate([y, -y]), min(ll_y, ur_y), max(ll_y, ur_y))
@@ -146,7 +159,6 @@ def get_geostationary_bounding_box(geos_area, nb_points=50):
 
 def get_area_slices(data_area, area_to_cover):
     """Compute the slice to read from an *area* based on an *area_to_cover*."""
-
     if data_area.proj_dict['proj'] != 'geos':
         raise NotImplementedError('Only geos supported')
 
@@ -191,11 +203,45 @@ def get_sub_area(area, xslice, yslice):
 
 
 def unzip_file(filename):
-    """Unzip the file if file is bzipped = ending with 'bz2'"""
-
+    """Unzip the file if file is bzipped = ending with 'bz2'."""
     if filename.endswith('bz2'):
-        bz2file = bz2.BZ2File(filename)
         fdn, tmpfilepath = tempfile.mkstemp()
+        LOGGER.info("Using temp file for BZ2 decompression: %s", tmpfilepath)
+        # If in python 3, try pbzip2
+        if sys.version_info.major >= 3:
+            pbzip = which('pbzip2')
+            # Run external pbzip2
+            if pbzip is not None:
+                n_thr = os.environ.get('OMP_NUM_THREADS')
+                if n_thr:
+                    runner = [pbzip,
+                              '-dc',
+                              '-p'+str(n_thr),
+                              filename]
+                else:
+                    runner = [pbzip,
+                              '-dc',
+                              filename]
+                p = Popen(runner, stdout=PIPE, stderr=PIPE)
+                stdout = BytesIO(p.communicate()[0])
+                status = p.returncode
+                if status != 0:
+                    raise IOError("pbzip2 error '%s', failed, status=%d"
+                                  % (filename, status))
+                with closing(os.fdopen(fdn, 'wb')) as ofpt:
+                    try:
+                        stdout.seek(0)
+                        shutil.copyfileobj(stdout, ofpt)
+                    except IOError:
+                        import traceback
+                        traceback.print_exc()
+                        LOGGER.info("Failed to read bzipped file %s",
+                                    str(filename))
+                        os.remove(tmpfilepath)
+                        raise
+                return tmpfilepath
+        # Otherwise, fall back to the original method
+        bz2file = bz2.BZ2File(filename)
         with closing(os.fdopen(fdn, 'wb')) as ofpt:
             try:
                 ofpt.write(bz2file.read())
@@ -205,19 +251,19 @@ def unzip_file(filename):
                 LOGGER.info("Failed to read bzipped file %s", str(filename))
                 os.remove(tmpfilepath)
                 return None
-
         return tmpfilepath
 
     return None
 
 
 def bbox(img):
-    """Find the bounding box around nonzero elements in the given array
+    """Find the bounding box around nonzero elements in the given array.
 
     Copied from https://stackoverflow.com/a/31402351/5703449 .
 
     Returns:
         rowmin, rowmax, colmin, colmax
+
     """
     rows = np.any(img, axis=1)
     cols = np.any(img, axis=0)
@@ -238,6 +284,7 @@ def get_earth_radius(lon, lat, a, b):
 
     Returns:
         Earth Radius (meters)
+
     """
     geocent = pyproj.Proj(proj='geocent', a=a, b=b, units='m')
     latlong = pyproj.Proj(proj='latlong', a=a, b=b, units='m')
@@ -246,7 +293,7 @@ def get_earth_radius(lon, lat, a, b):
 
 
 def reduce_mda(mda, max_size=100):
-    """Recursively remove arrays with more than `max_size` elements from the given metadata dictionary"""
+    """Recursively remove arrays with more than `max_size` elements from the given metadata dictionary."""
     reduced = {}
     for key, val in mda.items():
         if isinstance(val, dict):
