@@ -15,22 +15,29 @@
 #
 # You should have received a copy of the GNU General Public License along with
 # satpy.  If not, see <http://www.gnu.org/licenses/>.
+"""Interface to VIIRS flood product."""
+
 from satpy.readers.hdf4_utils import HDF4FileHandler
 from pyresample import geometry
 import numpy as np
 
 
 class VIIRSEDRFlood(HDF4FileHandler):
+    """VIIRS EDR Flood-product handler for HDF4 files."""
+
     @property
     def start_time(self):
+        """Get start time."""
         return self.filename_info['start_time']
 
     @property
     def end_time(self):
+        """Get end time."""
         return self.filename_info.get('end_time', self.start_time)
 
     @property
     def sensor_name(self):
+        """Get sensor name."""
         sensor = self['/attr/SensorIdentifyCode']
         if isinstance(sensor, np.ndarray):
             return str(sensor.astype(str)).lower()
@@ -38,12 +45,14 @@ class VIIRSEDRFlood(HDF4FileHandler):
 
     @property
     def platform_name(self):
+        """Get platform name."""
         platform_name = self['/attr/Satellitename']
         if isinstance(platform_name, np.ndarray):
             return str(platform_name.astype(str)).lower()
         return platform_name.lower()
 
     def get_metadata(self, data, ds_info):
+        """Get metadata."""
         metadata = {}
         metadata.update(data.attrs)
         metadata.update(ds_info)
@@ -57,6 +66,7 @@ class VIIRSEDRFlood(HDF4FileHandler):
         return metadata
 
     def get_dataset(self, ds_id, ds_info):
+        """Get dataset."""
         data = self[ds_id.name]
 
         data.attrs = self.get_metadata(data, ds_info)
@@ -73,6 +83,7 @@ class VIIRSEDRFlood(HDF4FileHandler):
         return data
 
     def get_area_def(self, ds_id):
+        """Get area definition."""
         data = self[ds_id.name]
 
         proj_dict = {
