@@ -15,12 +15,12 @@
 #
 # You should have received a copy of the GNU General Public License along with
 # satpy.  If not, see <http://www.gnu.org/licenses/>.
-"""The enhancements tests package.
-"""
+"""The enhancements tests package."""
 
 import sys
 
-from satpy.tests.enhancement_tests import test_enhancements, test_viirs
+from satpy.tests.enhancement_tests import (test_enhancements, test_viirs,
+                                           test_abi)
 
 if sys.version_info < (2, 7):
     import unittest2 as unittest
@@ -29,9 +29,10 @@ else:
 
 
 def suite():
-    """Test suite for all enhancement tests"""
+    """Create test suite for all enhancement tests."""
     mysuite = unittest.TestSuite()
     mysuite.addTests(test_enhancements.suite())
     mysuite.addTests(test_viirs.suite())
+    mysuite.addTests(test_abi.suite())
 
     return mysuite
