@@ -15,17 +15,11 @@
 #
 # You should have received a copy of the GNU General Public License along with
 # satpy.  If not, see <http://www.gnu.org/licenses/>.
-"""The abi_l1b reader tests package.
-"""
+"""The abi_l1b reader tests package."""
 
-import sys
 import numpy as np
 import xarray as xr
-
-if sys.version_info < (2, 7):
-    import unittest2 as unittest
-else:
-    import unittest
+import unittest
 
 try:
     from unittest import mock
@@ -33,30 +27,12 @@ except ImportError:
     import mock
 
 
-class FakeDataset(object):
-    def __init__(self, info, attrs):
-        for var_name, var_data in list(info.items()):
-            if isinstance(var_data, np.ndarray):
-                info[var_name] = xr.DataArray(var_data)
-        self.info = info
-        self.attrs = attrs
-
-    def __getitem__(self, key):
-        return self.info[key]
-
-    def rename(self, *args, **kwargs):
-        return self
-
-    def close(self):
-        return
-
-
 class Test_NC_ABI_L1B_Base(unittest.TestCase):
-    """Common setup for NC_ABI_L1B tests"""
+    """Common setup for NC_ABI_L1B tests."""
 
-    @mock.patch('satpy.readers.abi_l1b.xr')
+    @mock.patch('satpy.readers.abi_base.xr')
     def setUp(self, xr_, rad=None):
-        """Create a fake dataset using the given radiance data"""
+        """Create a fake dataset using the given radiance data."""
         from satpy.readers.abi_l1b import NC_ABI_L1B
 
         x_image = xr.DataArray(0.)
@@ -76,16 +52,18 @@ class Test_NC_ABI_L1B_Base(unittest.TestCase):
                     'units': 'W m-2 um-1 sr-1'
                 }
             )
-        rad['time'] = time
-        rad['x_image'] = x_image
-        rad['y_image'] = y_image
+        rad.coords['t'] = time
+        rad.coords['x_image'] = x_image
+        rad.coords['y_image'] = y_image
         x__ = xr.DataArray(
             range(5),
             attrs={'scale_factor': 2., 'add_offset': -1.},
+            dims=('x',)
         )
         y__ = xr.DataArray(
             range(2),
             attrs={'scale_factor': -2., 'add_offset': 1.},
+            dims=('y',)
         )
         proj = xr.DataArray(
             [],
@@ -98,30 +76,38 @@ class Test_NC_ABI_L1B_Base(unittest.TestCase):
                 'sweep_angle_axis': u'x'
             }
         )
-        yaw_flip = xr.DataArray([1])
-        xr_.open_dataset.return_value = FakeDataset({
-            'Rad': rad,
-            'band_id': np.array(8),
-            'x': x__,
-            'y': y__,
-            'x_image': x_image,
-            'y_image': y_image,
-            'goes_imager_projection': proj,
-            'yaw_flip_flag': yaw_flip,
-            "planck_fk1": np.array(13432.1),
-            "planck_fk2": np.array(1497.61),
-            "planck_bc1": np.array(0.09102),
-            "planck_bc2": np.array(0.99971),
-            "esun": np.array(2017),
-            "nominal_satellite_subpoint_lat": np.array(0.0),
-            "nominal_satellite_subpoint_lon": np.array(-89.5),
-            "nominal_satellite_height": np.array(35786.02),
-            "earth_sun_distance_anomaly_in_AU": np.array(0.99)},
-            {
+        fake_dataset = xr.Dataset(
+            data_vars={
+                'Rad': rad,
+                'band_id': np.array(8),
+                # 'x': x__,
+                # 'y': y__,
+                'x_image': x_image,
+                'y_image': y_image,
+                'goes_imager_projection': proj,
+                'yaw_flip_flag': np.array([1]),
+                "planck_fk1": np.array(13432.1),
+                "planck_fk2": np.array(1497.61),
+                "planck_bc1": np.array(0.09102),
+                "planck_bc2": np.array(0.99971),
+                "esun": np.array(2017),
+                "nominal_satellite_subpoint_lat": np.array(0.0),
+                "nominal_satellite_subpoint_lon": np.array(-89.5),
+                "nominal_satellite_height": np.array(35786.02),
+                "earth_sun_distance_anomaly_in_AU": np.array(0.99)
+            },
+            coords={
+                't': rad.coords['t'],
+                'x': x__,
+                'y': y__,
+
+            },
+            attrs={
                 "time_coverage_start": "2017-09-20T17:30:40.8Z",
                 "time_coverage_end": "2017-09-20T17:41:17.5Z",
-            })
-
+            },
+        )
+        xr_.open_dataset.return_value = fake_dataset
         self.reader = NC_ABI_L1B('filename',
                                  {'platform_shortname': 'G16', 'observation_type': 'Rad',
                                   'scene_abbr': 'C', 'scan_mode': 'M3'},
@@ -134,15 +120,13 @@ class Test_NC_ABI_L1B(Test_NC_ABI_L1B_Base):
     def test_basic_attributes(self):
         """Test getting basic file attributes."""
         from datetime import datetime
-        from satpy import DatasetID
         self.assertEqual(self.reader.start_time,
                          datetime(2017, 9, 20, 17, 30, 40, 800000))
         self.assertEqual(self.reader.end_time,
                          datetime(2017, 9, 20, 17, 41, 17, 500000))
-        self.assertEqual(self.reader.get_shape(DatasetID(name='C05'), {}),
-                         (2, 5))
 
     def test_get_dataset(self):
+        """Test the get_dataset method."""
         from satpy import DatasetID
         key = DatasetID(name='Rad', calibration='radiance')
         res = self.reader.get_dataset(key, {'info': 'info'})
@@ -154,7 +138,7 @@ class Test_NC_ABI_L1B(Test_NC_ABI_L1B_Base):
                'orbital_parameters': {'projection_altitude': 1.0,
                                       'projection_latitude': 0.0,
                                       'projection_longitude': -90.0,
-                                      'satellite_nominal_altitude': 35786.02,
+                                      'satellite_nominal_altitude': 35786020.,
                                       'satellite_nominal_latitude': 0.0,
                                       'satellite_nominal_longitude': -89.5,
                                       'yaw_flip': True},
@@ -162,16 +146,19 @@ class Test_NC_ABI_L1B(Test_NC_ABI_L1B_Base):
                'platform_name': 'GOES-16',
                'platform_shortname': 'G16',
                'production_site': None,
-               'satellite_altitude': 35786.02,
-               'satellite_latitude': 0.0,
-               'satellite_longitude': -89.5,
                'scan_mode': 'M3',
                'scene_abbr': 'C',
                'scene_id': None,
                'sensor': 'abi',
                'timeline_ID': None,
                'units': 'W m-2 um-1 sr-1'}
+
         self.assertDictEqual(res.attrs, exp)
+        # we remove any time dimension information
+        self.assertNotIn('t', res.coords)
+        self.assertNotIn('t', res.dims)
+        self.assertNotIn('time', res.coords)
+        self.assertNotIn('time', res.dims)
 
     def test_bad_calibration(self):
         """Test that asking for a bad calibration fails."""
@@ -179,7 +166,7 @@ class Test_NC_ABI_L1B(Test_NC_ABI_L1B_Base):
         self.assertRaises(ValueError, self.reader.get_dataset,
                           DatasetID(name='C05', calibration='_bad_'), {})
 
-    @mock.patch('satpy.readers.abi_l1b.geometry.AreaDefinition')
+    @mock.patch('satpy.readers.abi_base.geometry.AreaDefinition')
     def test_get_area_def(self, adef):
         """Test the area generation."""
         self.reader.get_area_def(None)
@@ -194,8 +181,10 @@ class Test_NC_ABI_L1B(Test_NC_ABI_L1B_Base):
 
 
 class Test_NC_ABI_L1B_ir_cal(Test_NC_ABI_L1B_Base):
+    """Test the NC_ABI_L1B reader's IR calibration."""
+
     def setUp(self):
-        """Setup for test."""
+        """Create fake data for the tests."""
         rad_data = (np.arange(10.).reshape((2, 5)) + 1.) * 50.
         rad_data = (rad_data + 1.) / 0.5
         rad_data = rad_data.astype(np.int16)
@@ -228,8 +217,10 @@ class Test_NC_ABI_L1B_ir_cal(Test_NC_ABI_L1B_Base):
 
 
 class Test_NC_ABI_L1B_vis_cal(Test_NC_ABI_L1B_Base):
+    """Test the NC_ABI_L1B reader."""
+
     def setUp(self):
-        """Setup for test."""
+        """Create fake data for the tests."""
         rad_data = (np.arange(10.).reshape((2, 5)) + 1.)
         rad_data = (rad_data + 1.) / 0.5
         rad_data = rad_data.astype(np.int16)
@@ -262,10 +253,14 @@ class Test_NC_ABI_L1B_vis_cal(Test_NC_ABI_L1B_Base):
 
 
 def suite():
-    """The test suite for test_scene."""
+    """Create test suite for test_scene."""
     loader = unittest.TestLoader()
     mysuite = unittest.TestSuite()
     mysuite.addTest(loader.loadTestsFromTestCase(Test_NC_ABI_L1B))
     mysuite.addTest(loader.loadTestsFromTestCase(Test_NC_ABI_L1B_ir_cal))
     mysuite.addTest(loader.loadTestsFromTestCase(Test_NC_ABI_L1B_vis_cal))
     return mysuite
+
+
+if __name__ == '__main__':
+    unittest.main()
