@@ -15,24 +15,15 @@
 #
 # You should have received a copy of the GNU General Public License along with
 # satpy.  If not, see <http://www.gnu.org/licenses/>.
-"""Module for testing the satpy.readers.geocat module.
-"""
+"""Module for testing the satpy.readers.geocat module."""
 
 import os
-import sys
 import numpy as np
 from satpy.tests.reader_tests.test_netcdf_utils import FakeNetCDF4FileHandler
 from satpy.tests.utils import convert_file_content_to_data_array
 
-if sys.version_info < (2, 7):
-    import unittest2 as unittest
-else:
-    import unittest
-
-try:
-    from unittest import mock
-except ImportError:
-    import mock
+import unittest
+from unittest import mock
 
 DEFAULT_FILE_DTYPE = np.uint16
 DEFAULT_FILE_SHAPE = (10, 300)
@@ -46,9 +37,10 @@ DEFAULT_LON_DATA = np.repeat([DEFAULT_LON_DATA], DEFAULT_FILE_SHAPE[0], axis=0)
 
 
 class FakeNetCDF4FileHandler2(FakeNetCDF4FileHandler):
-    """Swap-in NetCDF4 File Handler"""
+    """Swap-in NetCDF4 File Handler."""
+
     def get_test_content(self, filename, filename_info, filetype_info):
-        """Mimic reader input file content"""
+        """Mimic reader input file content."""
         file_content = {
             '/attr/Platform_Name': filename_info['platform_shortname'],
             '/attr/Element_Resolution': 2.,
@@ -67,7 +59,8 @@ class FakeNetCDF4FileHandler2(FakeNetCDF4FileHandler):
         }
         sensor = {
             'HIMAWARI-8': 'himawari8',
-            'GOES-16': 'goes16',
+            'GOES-17': 'goesr',
+            'GOES-16': 'goesr',
             'GOES-13': 'goes',
             'GOES-14': 'goes',
             'GOES-15': 'goes',
@@ -111,11 +104,12 @@ class FakeNetCDF4FileHandler2(FakeNetCDF4FileHandler):
 
 
 class TestGEOCATReader(unittest.TestCase):
-    """Test GEOCAT Reader"""
+    """Test GEOCAT Reader."""
+
     yaml_file = "geocat.yaml"
 
     def setUp(self):
-        """Wrap NetCDF4 file handler with our own fake handler"""
+        """Wrap NetCDF4 file handler with our own fake handler."""
         from satpy.config import config_search_paths
         from satpy.readers.geocat import GEOCATFileHandler
         self.reader_configs = config_search_paths(os.path.join('readers', self.yaml_file))
@@ -125,7 +119,7 @@ class TestGEOCATReader(unittest.TestCase):
         self.p.is_local = True
 
     def tearDown(self):
-        """Stop wrapping the NetCDF4 file handler"""
+        """Stop wrapping the NetCDF4 file handler."""
         self.p.stop()
 
     def test_init(self):
@@ -141,7 +135,7 @@ class TestGEOCATReader(unittest.TestCase):
         self.assertTrue(r.file_handlers)
 
     def test_load_all_old_goes(self):
-        """Test loading all test datasets"""
+        """Test loading all test datasets from old GOES files."""
         from satpy.readers import load_reader
         import xarray as xr
         r = load_reader(self.reader_configs)
@@ -160,8 +154,9 @@ class TestGEOCATReader(unittest.TestCase):
         self.assertIsNotNone(datasets['variable3'].attrs.get('flag_meanings'))
 
     def test_load_all_himawari8(self):
-        """Test loading all test datasets"""
+        """Test loading all test datasets from H8 NetCDF file."""
         from satpy.readers import load_reader
+        from pyresample.geometry import AreaDefinition
         import xarray as xr
         r = load_reader(self.reader_configs)
         with mock.patch('satpy.readers.geocat.netCDF4.Variable', xr.DataArray):
@@ -169,7 +164,6 @@ class TestGEOCATReader(unittest.TestCase):
                 'geocatL2.HIMAWARI-8.2017092.210730.R304.R20.nc',
             ])
             r.create_filehandlers(loadables)
-        # with mock.patch('satpy.readers.geocat.GEOCATFileHandler._load_nav', lambda self, x: self[x]):
         datasets = r.load(['variable1',
                            'variable2',
                            'variable3'])
@@ -178,10 +172,32 @@ class TestGEOCATReader(unittest.TestCase):
             self.assertIs(v.attrs['calibration'], None)
             self.assertEqual(v.attrs['units'], '1')
         self.assertIsNotNone(datasets['variable3'].attrs.get('flag_meanings'))
+        self.assertIsInstance(datasets['variable1'].attrs['area'], AreaDefinition)
+
+    def test_load_all_goes17_hdf4(self):
+        """Test loading all test datasets from GOES-17 HDF4 file."""
+        from satpy.readers import load_reader
+        from pyresample.geometry import AreaDefinition
+        import xarray as xr
+        r = load_reader(self.reader_configs)
+        with mock.patch('satpy.readers.geocat.netCDF4.Variable', xr.DataArray):
+            loadables = r.select_files_from_pathnames([
+                'geocatL2.GOES-17.CONUS.2020041.163130.hdf',
+            ])
+            r.create_filehandlers(loadables)
+        datasets = r.load(['variable1',
+                           'variable2',
+                           'variable3'])
+        self.assertEqual(len(datasets), 3)
+        for v in datasets.values():
+            self.assertIs(v.attrs['calibration'], None)
+            self.assertEqual(v.attrs['units'], '1')
+        self.assertIsNotNone(datasets['variable3'].attrs.get('flag_meanings'))
+        self.assertIsInstance(datasets['variable1'].attrs['area'], AreaDefinition)
 
 
 def suite():
-    """The test suite for test_viirs_l1b."""
+    """Create test suite for test_geocat."""
     loader = unittest.TestLoader()
     mysuite = unittest.TestSuite()
     mysuite.addTest(loader.loadTestsFromTestCase(TestGEOCATReader))
