@@ -142,12 +142,13 @@ class TestHRITFileHandler(unittest.TestCase):
 
     def test_get_area_def(self):
         area = self.reader.get_area_def('VIS06')
-        self.assertEqual(area.proj_dict, {'a': 6378169.0,
-                                          'b': 6356583.8,
-                                          'h': 35785831.0,
-                                          'lon_0': 44.0,
-                                          'proj': 'geos',
-                                          'units': 'm'})
+        proj_dict = area.proj_dict
+        self.assertEqual(proj_dict['a'], 6378169.0)
+        self.assertEqual(proj_dict['b'], 6356583.8)
+        self.assertEqual(proj_dict['h'], 35785831.0)
+        self.assertEqual(proj_dict['lon_0'], 44.0)
+        self.assertEqual(proj_dict['proj'], 'geos')
+        self.assertEqual(proj_dict['units'], 'm')
         self.assertEqual(area.area_extent,
                          (-77771774058.38356, -77771774058.38356,
                           30310525626438.438, 3720765401003.719))
