@@ -161,7 +161,7 @@ class TestSCMIFileHandlerArea(unittest.TestCase):
                                {'platform_shortname': 'G16'},
                                {'filetype': 'info'})
 
-    @mock.patch('satpy.readers.abi_l1b.geometry.AreaDefinition')
+    @mock.patch('satpy.readers.abi_base.geometry.AreaDefinition')
     def test_get_area_def_geos(self, adef):
         """Test the area generation for geos projection."""
         reader = self.create_reader(
@@ -186,7 +186,7 @@ class TestSCMIFileHandlerArea(unittest.TestCase):
         self.assertEqual(call_args[5], reader.nlines)
         np.testing.assert_allclose(call_args[6], (-2., -2., 2, 2.))
 
-    @mock.patch('satpy.readers.abi_l1b.geometry.AreaDefinition')
+    @mock.patch('satpy.readers.abi_base.geometry.AreaDefinition')
     def test_get_area_def_lcc(self, adef):
         """Test the area generation for lcc projection."""
         reader = self.create_reader(
@@ -211,7 +211,7 @@ class TestSCMIFileHandlerArea(unittest.TestCase):
         self.assertEqual(call_args[5], reader.nlines)
         np.testing.assert_allclose(call_args[6], (-2., -2., 2, 2.))
 
-    @mock.patch('satpy.readers.abi_l1b.geometry.AreaDefinition')
+    @mock.patch('satpy.readers.abi_base.geometry.AreaDefinition')
     def test_get_area_def_stere(self, adef):
         """Test the area generation for stere projection."""
         reader = self.create_reader(
@@ -236,7 +236,7 @@ class TestSCMIFileHandlerArea(unittest.TestCase):
         self.assertEqual(call_args[5], reader.nlines)
         np.testing.assert_allclose(call_args[6], (-2., -2., 2, 2.))
 
-    @mock.patch('satpy.readers.abi_l1b.geometry.AreaDefinition')
+    @mock.patch('satpy.readers.abi_base.geometry.AreaDefinition')
     def test_get_area_def_merc(self, adef):
         """Test the area generation for merc projection."""
         reader = self.create_reader(
@@ -260,7 +260,7 @@ class TestSCMIFileHandlerArea(unittest.TestCase):
         self.assertEqual(call_args[5], reader.nlines)
         np.testing.assert_allclose(call_args[6], (-2., -2., 2, 2.))
 
-    @mock.patch('satpy.readers.abi_l1b.geometry.AreaDefinition')
+    @mock.patch('satpy.readers.abi_base.geometry.AreaDefinition')
     def test_get_area_def_bad(self, adef):
         """Test the area generation for bad projection."""
         reader = self.create_reader(
