@@ -15,8 +15,7 @@
 #
 # You should have received a copy of the GNU General Public License along with
 # satpy.  If not, see <http://www.gnu.org/licenses/>.
-"""Unit tests for multiscene.py.
-"""
+"""Unit tests for multiscene.py."""
 
 import os
 import sys
@@ -75,12 +74,12 @@ def _create_test_dataset(name, shape=DEFAULT_SHAPE, area=None):
 
 
 def _create_test_scenes(num_scenes=2, shape=DEFAULT_SHAPE, area=None):
-    """Helper to create some test scenes."""
+    """Create some test scenes for various test cases."""
     from satpy import Scene
     ds1 = _create_test_dataset('ds1', shape=shape, area=area)
     ds2 = _create_test_dataset('ds2', shape=shape, area=area)
     scenes = []
-    for scn_idx in range(num_scenes):
+    for _ in range(num_scenes):
         scn = Scene()
         scn['ds1'] = ds1.copy()
         scn['ds2'] = ds2.copy()
@@ -153,11 +152,11 @@ class TestMultiSceneSave(unittest.TestCase):
     """Test saving a MultiScene to various formats."""
 
     def setUp(self):
-        """Create temporary directory to save files to"""
+        """Create temporary directory to save files to."""
         self.base_dir = tempfile.mkdtemp()
 
     def tearDown(self):
-        """Remove the temporary directory created for a test"""
+        """Remove the temporary directory created for a test."""
         try:
             shutil.rmtree(self.base_dir, ignore_errors=True)
         except OSError:
@@ -461,7 +460,7 @@ class TestBlendFuncs(unittest.TestCase):
 
 
 def suite():
-    """The test suite for test_multiscene."""
+    """Create the test suite for test_multiscene."""
     loader = unittest.TestLoader()
     mysuite = unittest.TestSuite()
     mysuite.addTest(loader.loadTestsFromTestCase(TestMultiScene))
