@@ -136,6 +136,7 @@ class TestUtils(unittest.TestCase):
         self.assertAlmostEqual(z__, sqrt(1) / 2)
 
     def test_xyz2lonlat(self):
+        """Test xyz2lonlat."""
         lon, lat = xyz2lonlat(1, 0, 0)
         self.assertAlmostEqual(lon, 0)
         self.assertAlmostEqual(lat, 0)
@@ -144,6 +145,10 @@ class TestUtils(unittest.TestCase):
         self.assertAlmostEqual(lon, 90)
         self.assertAlmostEqual(lat, 0)
 
+        lon, lat = xyz2lonlat(0, 0, 1, asin=True)
+        self.assertAlmostEqual(lon, 0)
+        self.assertAlmostEqual(lat, 90)
+
         lon, lat = xyz2lonlat(0, 0, 1)
         self.assertAlmostEqual(lon, 0)
         self.assertAlmostEqual(lat, 90)
@@ -153,6 +158,7 @@ class TestUtils(unittest.TestCase):
         self.assertAlmostEqual(lat, 0)
 
     def test_xyz2angle(self):
+        """Test xyz2angle."""
         azi, zen = xyz2angle(1, 0, 0)
         self.assertAlmostEqual(azi, 90)
         self.assertAlmostEqual(zen, 90)
@@ -165,6 +171,10 @@ class TestUtils(unittest.TestCase):
         self.assertAlmostEqual(azi, 0)
         self.assertAlmostEqual(zen, 0)
 
+        azi, zen = xyz2angle(0, 0, 1, acos=True)
+        self.assertAlmostEqual(azi, 0)
+        self.assertAlmostEqual(zen, 0)
+
         azi, zen = xyz2angle(sqrt(2) / 2, sqrt(2) / 2, 0)
         self.assertAlmostEqual(azi, 45)
         self.assertAlmostEqual(zen, 90)
@@ -178,6 +188,7 @@ class TestUtils(unittest.TestCase):
         self.assertAlmostEqual(zen, 90)
 
     def test_proj_units_to_meters(self):
+        """Test proj units to meters conversion."""
         prj = '+asd=123123123123'
         res = proj_units_to_meters(prj)
         self.assertEqual(res, prj)
@@ -196,6 +207,7 @@ class TestUtils(unittest.TestCase):
 
     @mock.patch('satpy.utils.warnings.warn')
     def test_get_satpos(self, warn_mock):
+        """Test getting the satellite position."""
         orb_params = {'nadir_longitude': 1,
                       'satellite_actual_longitude': 1.1,
                       'satellite_nominal_longitude': 1.2,
@@ -244,8 +256,7 @@ class TestUtils(unittest.TestCase):
 
 
 def suite():
-    """The test suite.
-    """
+    """Test suite."""
     loader = unittest.TestLoader()
     mysuite = unittest.TestSuite()
     mysuite.addTest(loader.loadTestsFromTestCase(TestUtils))
