@@ -220,8 +220,10 @@ class TestCFWriter(unittest.TestCase):
                                                     end_time=end_time))
         with TempFile() as filename:
             scn.save_datasets(filename=filename, writer='cf')
-            with xr.open_dataset(filename, decode_cf=False) as f:
-                self.assertTrue(np.all(f['time_bnds'][:] == np.array([-300.,  600.])))
+            with xr.open_dataset(filename, decode_cf=True) as f:
+                np.testing.assert_array_equal(f['time'], scn['test-array']['time'])
+                bounds_exp = np.array([[start_time, end_time]], dtype='datetime64[m]')
+                np.testing.assert_array_equal(f['time_bnds'], bounds_exp)
 
     def test_bounds(self):
         """Test setting time bounds."""
@@ -238,8 +240,23 @@ class TestCFWriter(unittest.TestCase):
                                                     end_time=end_time))
         with TempFile() as filename:
             scn.save_datasets(filename=filename, writer='cf')
+            # Check decoded time coordinates & bounds
+            with xr.open_dataset(filename, decode_cf=True) as f:
+                bounds_exp = np.array([[start_time, end_time]], dtype='datetime64[m]')
+                np.testing.assert_array_equal(f['time_bnds'], bounds_exp)
+                self.assertEqual(f['time'].attrs['bounds'], 'time_bnds')
+
+            # Check raw time coordinates & bounds
             with xr.open_dataset(filename, decode_cf=False) as f:
-                self.assertTrue(np.all(f['time_bnds'][:] == np.array([-300.,  600.])))
+                np.testing.assert_almost_equal(f['time_bnds'], [[-0.0034722, 0.0069444]])
+
+        # User-specified time encoding should have preference
+        with TempFile() as filename:
+            time_units = 'seconds since 2018-01-01'
+            scn.save_datasets(filename=filename, encoding={'time': {'units': time_units}},
+                              writer='cf')
+            with xr.open_dataset(filename, decode_cf=False) as f:
+                np.testing.assert_array_equal(f['time_bnds'], [[12909600, 12910500]])
 
     def test_bounds_minimum(self):
         """Test minimum bounds."""
@@ -264,8 +281,9 @@ class TestCFWriter(unittest.TestCase):
                                                      end_time=end_timeB))
         with TempFile() as filename:
             scn.save_datasets(filename=filename, writer='cf')
-            with xr.open_dataset(filename, decode_cf=False) as f:
-                self.assertTrue(np.all(f['time_bnds'][:] == np.array([-300.,  600.])))
+            with xr.open_dataset(filename, decode_cf=True) as f:
+                bounds_exp = np.array([[start_timeA, end_timeB]], dtype='datetime64[m]')
+                np.testing.assert_array_equal(f['time_bnds'], bounds_exp)
 
     def test_bounds_missing_time_info(self):
         """Test time bounds generation in case of missing time."""
@@ -286,11 +304,12 @@ class TestCFWriter(unittest.TestCase):
                                           coords={'time': [np.datetime64('2018-05-30T10:05:00')]})
         with TempFile() as filename:
             scn.save_datasets(filename=filename, writer='cf')
-            with xr.open_dataset(filename, decode_cf=False) as f:
-                self.assertTrue(np.all(f['time_bnds'][:] == np.array([-300.,  600.])))
+            with xr.open_dataset(filename, decode_cf=True) as f:
+                bounds_exp = np.array([[start_timeA, end_timeA]], dtype='datetime64[m]')
+                np.testing.assert_array_equal(f['time_bnds'], bounds_exp)
 
     def test_encoding_kwarg(self):
-        """Test encoding of keyword arguments."""
+        """Test 'encoding' keyword argument."""
         from satpy import Scene
         import xarray as xr
         scn = Scene()
@@ -312,6 +331,24 @@ class TestCFWriter(unittest.TestCase):
                 # check that dtype behave as int8
                 self.assertTrue(np.iinfo(f['test-array'][:].dtype).max == 127)
 
+    def test_unlimited_dims_kwarg(self):
+        """Test specification of unlimited dimensions."""
+        from satpy import Scene
+        import xarray as xr
+        scn = Scene()
+        start_time = datetime(2018, 5, 30, 10, 0)
+        end_time = datetime(2018, 5, 30, 10, 15)
+        test_array = np.array([[1, 2], [3, 4]])
+        scn['test-array'] = xr.DataArray(test_array,
+                                         dims=['x', 'y'],
+                                         coords={'time': np.datetime64('2018-05-30T10:05:00')},
+                                         attrs=dict(start_time=start_time,
+                                                    end_time=end_time))
+        with TempFile() as filename:
+            scn.save_datasets(filename=filename, writer='cf', unlimited_dims=['time'])
+            with xr.open_dataset(filename) as f:
+                self.assertSetEqual(f.encoding['unlimited_dims'], {'time'})
+
     def test_header_attrs(self):
         """Check master attributes are set."""
         from satpy import Scene
@@ -324,14 +361,36 @@ class TestCFWriter(unittest.TestCase):
                                                     end_time=end_time))
         with TempFile() as filename:
             header_attrs = {'sensor': 'SEVIRI',
-                            'orbit': None}
+                            'orbit': 99999,
+                            'none': None,
+                            'list': [1, 2, 3],
+                            'set': {1, 2, 3},
+                            'dict': {'a': 1, 'b': 2},
+                            'nested': {'outer': {'inner1': 1, 'inner2': 2}},
+                            'bool': True,
+                            'bool_': np.bool_(True)}
             scn.save_datasets(filename=filename,
                               header_attrs=header_attrs,
+                              flatten_attrs=True,
                               writer='cf')
             with xr.open_dataset(filename) as f:
-                self.assertTrue(f.attrs['sensor'] == 'SEVIRI')
-                self.assertTrue('sensor' in f.attrs.keys())
-                self.assertTrue('orbit' not in f.attrs.keys())
+                self.assertIn('history', f.attrs)
+                self.assertEqual(f.attrs['sensor'], 'SEVIRI')
+                self.assertEqual(f.attrs['orbit'], 99999)
+                np.testing.assert_array_equal(f.attrs['list'], [1, 2, 3])
+                if sys.version_info.major == 3:
+                    self.assertEqual(f.attrs['set'], '{1, 2, 3}')
+                else:
+                    # json module seems to encode sets differently in
+                    # Python 2 and 3
+                    self.assertEqual(f.attrs['set'], u'set([1, 2, 3])')
+                self.assertEqual(f.attrs['dict_a'], 1)
+                self.assertEqual(f.attrs['dict_b'], 2)
+                self.assertEqual(f.attrs['nested_outer_inner1'], 1)
+                self.assertEqual(f.attrs['nested_outer_inner2'], 2)
+                self.assertEqual(f.attrs['bool'], 'true')
+                self.assertEqual(f.attrs['bool_'], 'true')
+                self.assertTrue('none' not in f.attrs.keys())
 
     def get_test_attrs(self):
         """Create some dataset attributes for testing purpose.
@@ -345,6 +404,13 @@ class TestCFWriter(unittest.TestCase):
                  'end_time': datetime(2018, 1, 1, 0, 15),
                  'int': 1,
                  'float': 1.0,
+                 'none': None,  # should be dropped
+                 'numpy_int': np.uint8(1),
+                 'numpy_float': np.float32(1),
+                 'numpy_bool': np.bool(True),
+                 'numpy_void': np.void(0),
+                 'numpy_bytes': np.bytes_('test'),
+                 'numpy_string': np.string_('test'),
                  'list': [1, 2, np.float64(3)],
                  'nested_list': ["1", ["2", [3]]],
                  'bool': True,
@@ -364,34 +430,46 @@ class TestCFWriter(unittest.TestCase):
                    'end_time': '2018-01-01 00:15:00',
                    'int': 1,
                    'float': 1.0,
-                   'list': [1, 2, 3.0],
+                   'numpy_int': np.uint8(1),
+                   'numpy_float': np.float32(1),
+                   'numpy_bool': 'true',
+                   'numpy_void': '[]',
+                   'numpy_bytes': 'test',
+                   'numpy_string': 'test',
+                   'list': [1, 2, np.float64(3)],
                    'nested_list': '["1", ["2", [3]]]',
-                   'bool': 'True',
+                   'bool': 'true',
                    'array': np.array([1, 2, 3], dtype='uint8'),
-                   'array_bool': ['True', 'False', 'True'],
+                   'array_bool': ['true', 'false', 'true'],
                    'array_2d': '[[1, 2], [3, 4]]',
                    'array_3d': '[[[1, 2], [3, 4]], [[1, 2], [3, 4]]]',
                    'dict': '{"a": 1, "b": 2}',
                    'nested_dict': '{"l1": {"l2": {"l3": [1, 2, 3]}}}',
                    'raw_metadata': '{"recarray": [[0, 0], [0, 0], [0, 0]], '
-                                   '"flag": "True", "dict": {"a": 1, "b": [1, 2, 3]}}'}
+                                   '"flag": "true", "dict": {"a": 1, "b": [1, 2, 3]}}'}
         encoded_flat = {'name': 'IR_108',
                         'start_time': '2018-01-01 00:00:00',
                         'end_time': '2018-01-01 00:15:00',
                         'int': 1,
                         'float': 1.0,
-                        'list': [1, 2, 3.0],
+                        'numpy_int': np.uint8(1),
+                        'numpy_float': np.float32(1),
+                        'numpy_bool': 'true',
+                        'numpy_void': '[]',
+                        'numpy_bytes': 'test',
+                        'numpy_string': 'test',
+                        'list': [1, 2, np.float64(3)],
                         'nested_list': '["1", ["2", [3]]]',
-                        'bool': 'True',
+                        'bool': 'true',
                         'array': np.array([1, 2, 3], dtype='uint8'),
-                        'array_bool': ['True', 'False', 'True'],
+                        'array_bool': ['true', 'false', 'true'],
                         'array_2d': '[[1, 2], [3, 4]]',
                         'array_3d': '[[[1, 2], [3, 4]], [[1, 2], [3, 4]]]',
                         'dict_a': 1,
                         'dict_b': 2,
                         'nested_dict_l1_l2_l3': np.array([1, 2, 3], dtype='uint8'),
                         'raw_metadata_recarray': '[[0, 0], [0, 0], [0, 0]]',
-                        'raw_metadata_flag': 'True',
+                        'raw_metadata_flag': 'true',
                         'raw_metadata_dict_a': 1,
                         'raw_metadata_dict_b': np.array([1, 2, 3], dtype='uint8')}
         return attrs, encoded, encoded_flat
@@ -406,6 +484,9 @@ class TestCFWriter(unittest.TestCase):
                 self.assertEqual(val1.dtype, val2.dtype)
             else:
                 self.assertEqual(val1, val2)
+                if isinstance(val1, (np.floating, np.integer, np.bool_)):
+                    self.assertTrue(isinstance(val2, np.generic))
+                    self.assertEqual(val1.dtype, val2.dtype)
 
     def test_encode_attrs_nc(self):
         """Test attributes encoding."""
@@ -416,11 +497,11 @@ class TestCFWriter(unittest.TestCase):
 
         # Test encoding
         encoded = encode_attrs_nc(attrs)
-        self.assertDictWithArraysEqual(encoded, expected)
+        self.assertDictWithArraysEqual(expected, encoded)
 
         # Test decoding of json-encoded attributes
         raw_md_roundtrip = {'recarray': [[0, 0], [0, 0], [0, 0]],
-                            'flag': "True",
+                            'flag': 'true',
                             'dict': {'a': 1, 'b': [1, 2, 3]}}
         self.assertDictEqual(json.loads(encoded['raw_metadata']), raw_md_roundtrip)
         self.assertListEqual(json.loads(encoded['array_3d']), [[[1, 2], [3, 4]], [[1, 2], [3, 4]]])
@@ -662,6 +743,15 @@ class TestCFWriter(unittest.TestCase):
         import pyresample.geometry
         from satpy.writers.cf_writer import area2gridmapping
 
+        def _gm_matches(gmapping, expected):
+            """Assert that all keys in ``expected`` match the values in ``gmapping``."""
+            for attr_key, attr_val in expected.attrs.items():
+                test_val = gmapping.attrs[attr_key]
+                if attr_val is None or isinstance(attr_val, str):
+                    self.assertEqual(test_val, attr_val)
+                else:
+                    np.testing.assert_almost_equal(test_val, attr_val, decimal=3)
+
         ds_base = xr.DataArray(data=[[1, 2], [3, 4]], dims=('y', 'x'), coords={'y': [1, 2], 'x': [3, 4]},
                                attrs={'name': 'var1'})
 
@@ -678,8 +768,8 @@ class TestCFWriter(unittest.TestCase):
             area_extent=[-1, -1, 1, 1])
         geos_expected = xr.DataArray(data=0,
                                      attrs={'perspective_point_height': h,
-                                            'latitude_of_projection_origin': None,
-                                            'longitude_of_projection_origin': None,
+                                            'latitude_of_projection_origin': 0,
+                                            'longitude_of_projection_origin': 0,
                                             'grid_mapping_name': 'geostationary',
                                             'semi_major_axis': a,
                                             'semi_minor_axis': b,
@@ -691,7 +781,7 @@ class TestCFWriter(unittest.TestCase):
         res, grid_mapping = area2gridmapping(ds)
 
         self.assertEqual(res.attrs['grid_mapping'], 'geos')
-        self.assertEqual(grid_mapping, geos_expected)
+        _gm_matches(grid_mapping, geos_expected)
 
         # b) Projection does not have a corresponding CF representation (COSMO)
         cosmo7 = pyresample.geometry.AreaDefinition(
@@ -703,8 +793,6 @@ class TestCFWriter(unittest.TestCase):
             width=597, height=510,
             area_extent=[-1812933, -1003565, 814056, 1243448]
         )
-        proj_str = '+proj=ob_tran +ellps=WGS84 +lat_0=46.0 +lon_0=4.535 +o_proj=stere +o_lat_p=90.0 +o_lon_p=-5.465'
-        cosmo_expected = xr.DataArray(data=0, attrs={'name': 'proj4', 'proj4': proj_str})
 
         ds = ds_base.copy()
         ds.attrs['area'] = cosmo7
@@ -712,9 +800,102 @@ class TestCFWriter(unittest.TestCase):
         with mock.patch('satpy.writers.cf_writer.warnings.warn') as warn:
             res, grid_mapping = area2gridmapping(ds)
             warn.assert_called()
-            self.assertDictEqual(dict(pyresample.geometry.proj4_str_to_dict(res.attrs['grid_proj4'])),
-                                 dict(pyresample.geometry.proj4_str_to_dict(proj_str)))
-            self.assertEqual(grid_mapping, cosmo_expected)
+            proj_dict = pyresample.geometry.proj4_str_to_dict(res.attrs['grid_proj4'])
+            self.assertEqual(proj_dict['lon_0'], 4.535)
+            self.assertEqual(proj_dict['lat_0'], 46.0)
+            self.assertEqual(proj_dict['o_lon_p'], -5.465)
+            self.assertEqual(proj_dict['o_lat_p'], 90.0)
+            self.assertEqual(proj_dict['proj'], 'ob_tran')
+            self.assertEqual(proj_dict['o_proj'], 'stere')
+            self.assertEqual(proj_dict['ellps'], 'WGS84')
+            self.assertEqual(grid_mapping.attrs['name'], 'proj4')
+
+        # c) Projection Transverse Mercator
+        lat_0 = 36.5
+        lon_0 = 15.0
+
+        tmerc = pyresample.geometry.AreaDefinition(
+            area_id='tmerc',
+            description='tmerc',
+            proj_id='tmerc',
+            projection={'proj': 'tmerc', 'ellps': 'WGS84', 'lat_0': 36.5, 'lon_0': 15.0},
+            width=2, height=2,
+            area_extent=[-1, -1, 1, 1])
+
+        tmerc_expected = xr.DataArray(data=0,
+                                      attrs={'latitude_of_projection_origin': lat_0,
+                                             'longitude_of_central_meridian': lon_0,
+                                             'grid_mapping_name': 'transverse_mercator',
+                                             'reference_ellipsoid_name': 'WGS84',
+                                             'false_easting': 0.,
+                                             'false_northing': 0.,
+                                             'name': 'tmerc'})
+
+        ds = ds_base.copy()
+        ds.attrs['area'] = tmerc
+        res, grid_mapping = area2gridmapping(ds)
+        self.assertEqual(res.attrs['grid_mapping'], 'tmerc')
+        _gm_matches(grid_mapping, tmerc_expected)
+
+        # d) Projection that has a representation but no explicit a/b
+        h = 35785831.
+        geos = pyresample.geometry.AreaDefinition(
+            area_id='geos',
+            description='geos',
+            proj_id='geos',
+            projection={'proj': 'geos', 'h': h, 'datum': 'WGS84', 'ellps': 'GRS80'},
+            width=2, height=2,
+            area_extent=[-1, -1, 1, 1])
+        geos_expected = xr.DataArray(data=0,
+                                     attrs={'perspective_point_height': h,
+                                            'latitude_of_projection_origin': 0,
+                                            'longitude_of_projection_origin': 0,
+                                            'grid_mapping_name': 'geostationary',
+                                            'semi_major_axis': 6378137.0,
+                                            'semi_minor_axis': 6356752.314,
+                                            'sweep_axis': None,
+                                            'name': 'geos'})
+
+        ds = ds_base.copy()
+        ds.attrs['area'] = geos
+        res, grid_mapping = area2gridmapping(ds)
+
+        self.assertEqual(res.attrs['grid_mapping'], 'geos')
+        _gm_matches(grid_mapping, geos_expected)
+
+        # e) oblique Mercator
+        area = pyresample.geometry.AreaDefinition(
+            area_id='omerc_otf',
+            description='On-the-fly omerc area',
+            proj_id='omerc',
+            projection={'alpha': '9.02638777018478', 'ellps': 'WGS84', 'gamma': '0', 'k': '1',
+                        'lat_0': '-0.256794486098476', 'lonc': '13.7888658224205',
+                        'proj': 'omerc', 'units': 'm'},
+            width=2837,
+            height=5940,
+            area_extent=[-1460463.0893, 3455291.3877, 1538407.1158, 9615788.8787]
+        )
+
+        omerc_dict = {'name': 'omerc',
+                      'azimuth_of_central_line': 9.02638777018478,
+                      'false_easting': 0.,
+                      'false_northing': 0.,
+                      'gamma': 0,
+                      'geographic_crs_name': "unknown",
+                      'grid_mapping_name': "oblique_mercator",
+                      'horizontal_datum_name': "unknown",
+                      'latitude_of_projection_origin': -0.256794486098476,
+                      'longitude_of_projection_origin': 13.7888658224205,
+                      'prime_meridian_name': "Greenwich",
+                      'reference_ellipsoid_name': "WGS84"}
+        omerc_expected = xr.DataArray(data=0, attrs=omerc_dict)
+
+        ds = ds_base.copy()
+        ds.attrs['area'] = area
+        res, grid_mapping = area2gridmapping(ds)
+
+        self.assertEqual(res.attrs['grid_mapping'], 'omerc')
+        _gm_matches(grid_mapping, omerc_expected)
 
     def test_area2lonlat(self):
         """Test the conversion from areas to lon/lat."""
